@@ -20,12 +20,11 @@ namespace Mono.Debugger.Backend
 	{
 		protected IntPtr server_handle;
 		protected IntPtr io_data;
-		protected Bfd bfd;
+		protected NativeExecutableReader exe;
 		protected ThreadManager thread_manager;
 
 		protected readonly ProcessStart start;
 
-		protected readonly BfdContainer bfd_container;
 		protected readonly ProcessServant process;
 		protected readonly DebuggerErrorHandler error_handler;
 		protected readonly BreakpointManager breakpoint_manager;
@@ -43,6 +42,12 @@ namespace Mono.Debugger.Backend
 		bool has_signals;
 		SignalInfo signal_info;
 
+		public static bool IsRunningOnWindows {
+			get {
+				return ((int)Environment.OSVersion.Platform < 4);
+			}
+		}
+
 		public bool HasTarget {
 			get {
 				return has_target;
@@ -60,7 +65,7 @@ namespace Mono.Debugger.Backend
 		static extern TargetError mono_debugger_server_initialize_process (IntPtr handle);
 
 		[DllImport("monodebuggerserver")]
-		static extern TargetError mono_debugger_server_initialize_thread (IntPtr handle, int child_pid);
+		static extern TargetError mono_debugger_server_initialize_thread (IntPtr handle, int child_pid, bool wait);
 
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_io_thread_main (IntPtr io_data, ChildOutputHandler output_handler);
@@ -173,7 +178,7 @@ namespace Mono.Debugger.Backend
 		static extern TargetError mono_debugger_server_get_threads (IntPtr handle, out int count, out IntPtr data);
 
 		[DllImport("monodebuggerserver")]
-		static extern TargetError mono_debugger_server_get_application (IntPtr handle, out string exe_file, out string cwd, out int nargs, out IntPtr data);
+		static extern TargetError mono_debugger_server_get_application (IntPtr handle, out IntPtr exe_file, out IntPtr cwd, out int nargs, out IntPtr data);
 
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_detach_after_fork (IntPtr handle);
@@ -187,9 +192,18 @@ namespace Mono.Debugger.Backend
 		[DllImport("monodebuggerserver")]
 		static extern TargetError mono_debugger_server_get_callback_frame (IntPtr handle, long stack_pointer, bool exact_match, IntPtr info);
 
+		[DllImport("monodebuggerserver")]
+		static extern TargetError mono_debugger_server_restart_notification (IntPtr handle);
+
 		[DllImport("monodebuggerserver")]
 		static extern void mono_debugger_server_set_runtime_info (IntPtr handle, IntPtr mono_runtime_info);
 
+		[DllImport("monodebuggerserver")]
+		static extern ServerType mono_debugger_server_get_server_type ();
+
+		[DllImport("monodebuggerserver")]
+		static extern ServerCapabilities mono_debugger_server_get_capabilities ();
+
 		internal enum ChildEventType {
 			NONE = 0,
 			UNKNOWN_ERROR = 1,
@@ -221,6 +235,19 @@ namespace Mono.Debugger.Backend
 			WRITE
 		}
 
+		internal enum ServerType {
+			UNKNOWN = 0,
+			LINUX_PTRACE = 1,
+			DARWIN = 2,
+			WINDOWS = 3
+		}
+
+		internal enum ServerCapabilities {
+			NONE = 0,
+			THREAD_EVENTS = 1,
+			CAN_DETACH_ANY = 2
+		}
+
 		internal delegate void ChildEventHandler (ChildEventType message, int arg);
 
 		internal sealed class ChildEvent
@@ -269,6 +296,7 @@ namespace Mono.Debugger.Backend
 			public int SIGSEGV;
 			public int SIGILL;
 			public int SIGBUS;
+			public int SIGWINCH;
 
 			public int Kernel_SIGRTMIN;
 			public int MonoThreadAbortSignal;
@@ -290,7 +318,6 @@ namespace Mono.Debugger.Backend
 			this.process = process;
 			this.start = start;
 			this.native = start.IsNative;
-			this.bfd_container = process.BfdContainer;
 			this.error_handler = error_handler;
 			this.breakpoint_manager = bpm;
 			this.address_domain = address_domain;
@@ -308,19 +335,26 @@ namespace Mono.Debugger.Backend
 				thread_manager.AddressDomain);
 		}
 
-		public Inferior CreateThread ()
+		public Inferior CreateThread (int pid, bool do_attach)
 		{
 			Inferior inferior = new Inferior (
 				thread_manager, process, start, breakpoint_manager,
 				error_handler, address_domain);
 
+			inferior.child_pid = pid;
+
 			inferior.signal_info = signal_info;
 			inferior.has_signals = has_signals;
 
 			inferior.target_info = target_info;
-			inferior.bfd = bfd;
+			inferior.exe = exe;
 
-			inferior.arch = inferior.bfd.Architecture;
+			inferior.arch = inferior.process.Architecture;
+
+			if (do_attach)
+				inferior.Attach (pid);
+			else
+				inferior.InitializeThread (pid);
 
 			return inferior;
 		}
@@ -575,18 +609,17 @@ namespace Mono.Debugger.Backend
 				server_handle, breakpoint));
 		}
 
+		public void RestartNotification ()
+		{
+			check_error (mono_debugger_server_restart_notification (server_handle));
+		}
+
 		public ProcessStart ProcessStart {
 			get {
 				return start;
 			}
 		}
 
-		public BfdContainer BfdContainer {
-			get {
-				return bfd_container;
-			}
-		}
-
 		public ProcessServant Process {
 			get {
 				return process;
@@ -607,9 +640,13 @@ namespace Mono.Debugger.Backend
 
 			IntPtr error;
 
+			string[] args = new string[start.CommandLineArguments.Length + 1];
+			Array.Copy(start.CommandLineArguments, args, start.CommandLineArguments.Length);
+			string[] env = new string[start.Environment.Length + 1];
+			Array.Copy(start.Environment, env, start.Environment.Length);
 			TargetError result = mono_debugger_server_spawn (
-				server_handle, start.WorkingDirectory, start.CommandLineArguments,
-				start.Environment, start.RedirectOutput, out child_pid, out io_data, out error);
+				server_handle, start.WorkingDirectory, args,
+				env, start.RedirectOutput, out child_pid, out io_data, out error);
 			if (result != TargetError.None) {
 				string message = Marshal.PtrToStringAuto (error);
 				g_free (error);
@@ -624,14 +661,6 @@ namespace Mono.Debugger.Backend
 				io_thread.Start ();
 			}
 
-			return child_pid;
-		}
-
-		public void InitializeProcess ()
-		{
-			if (initialized)
-				throw new TargetException (TargetError.AlreadyHaveTarget);
-
 			initialized = true;
 
 			check_error (mono_debugger_server_initialize_process (server_handle));
@@ -639,6 +668,8 @@ namespace Mono.Debugger.Backend
 			SetupInferior ();
 
 			change_target_state (TargetState.Stopped, 0);
+
+			return child_pid;
 		}
 
 		public void InitializeThread (int pid)
@@ -648,7 +679,9 @@ namespace Mono.Debugger.Backend
 
 			initialized = true;
 
-			check_error (mono_debugger_server_initialize_thread (server_handle, pid));
+			bool pending_sigstop = process.ThreadManager.HasPendingSigstopForNewThread (pid);
+
+			check_error (mono_debugger_server_initialize_thread (server_handle, pid, !pending_sigstop));
 			this.child_pid = pid;
 
 			SetupInferior ();
@@ -656,9 +689,30 @@ namespace Mono.Debugger.Backend
 			change_target_state (TargetState.Stopped, 0);
 		}
 
+		public void InitializeAfterExec (int pid)
+		{
+			if (initialized)
+				throw new TargetException (TargetError.AlreadyHaveTarget);
+
+			initialized = true;
+
+			check_error (mono_debugger_server_initialize_thread (server_handle, pid, false));
+			this.child_pid = pid;
+
+			string exe_file, cwd;
+			string[] cmdline_args;
+			exe_file = GetApplication (out cwd, out cmdline_args);
+
+			start.SetupApplication (exe_file, cwd, cmdline_args);
+
+			SetupInferior ();
+
+			change_target_state (TargetState.Stopped, 0);
+		}
+
 		public void Attach (int pid)
 		{
-			if (has_target)
+			if (has_target || initialized)
 				throw new TargetException (TargetError.AlreadyHaveTarget);
 
 			has_target = true;
@@ -672,22 +726,13 @@ namespace Mono.Debugger.Backend
 
 			start.SetupApplication (exe_file, cwd, cmdline_args);
 
+			initialized = true;
+
 			SetupInferior ();
 
 			change_target_state (TargetState.Stopped, 0);
 		}
 
-		void child_execd ()
-		{
-			string exe_file, cwd;
-			string[] cmdline_args;
-			exe_file = GetApplication (out cwd, out cmdline_args);
-
-			start.SetupApplication (exe_file, cwd, cmdline_args);
-
-			SetupInferior ();
-		}
-
 		public ChildEvent ProcessEvent (int status)
 		{
 			long arg, data1, data2;
@@ -712,11 +757,11 @@ namespace Mono.Debugger.Backend
 			case ChildEventType.CHILD_STOPPED:
 			case ChildEventType.CHILD_INTERRUPTED:
 			case ChildEventType.CHILD_HIT_BREAKPOINT:
+			case ChildEventType.CHILD_NOTIFICATION:
 				change_target_state (TargetState.Stopped);
 				break;
 
 			case ChildEventType.CHILD_EXECD:
-				child_execd ();
 				break;
 			}
 
@@ -785,9 +830,13 @@ namespace Mono.Debugger.Backend
 			target_info = GetTargetMemoryInfo (address_domain);
 
 			try {
-				bfd = bfd_container.AddFile (
-					target_info, start.TargetApplication, TargetAddress.Null,
-					start.LoadNativeSymbolTable, true);
+				string cwd;
+				string[] cmdline_args;
+
+				string application = GetApplication (out cwd, out cmdline_args);
+
+				exe = process.OperatingSystem.LoadExecutable (
+					target_info, application, start.LoadNativeSymbolTable);
 			} catch (Exception e) {
 				if (error_handler != null)
 					error_handler (this, String.Format (
@@ -798,37 +847,30 @@ namespace Mono.Debugger.Backend
 				return;
 			}
 
-			bfd_container.SetupInferior (target_info, bfd);
-
-			arch = bfd.Architecture;
-		}
-
-		public void InitializeModules ()
-		{
-			bfd.UpdateSharedLibraryInfo (this, this);
+			arch = process.Architecture;
 		}
 
 		public BreakpointManager BreakpointManager {
 			get { return breakpoint_manager; }
 		}
 
-		public Bfd Bfd {
-			get { return bfd; }
+		public NativeExecutableReader Executable {
+			get { return exe; }
 		}
 
 		public TargetAddress SimpleLookup (string name)
 		{
-			return bfd [name];
+			return exe.LookupSymbol (name);
 		}
 
 		public TargetAddress GetSectionAddress (string name)
 		{
-			return bfd.GetSectionAddress (name);
+			return exe.GetSectionAddress (name);
 		}
 
-		public TargetAddress MainMethodAddress {
+		public TargetAddress EntryPoint {
 			get {
-				return bfd.EntryPoint;
+				return exe.EntryPoint;
 			}
 		}
 
@@ -1186,6 +1228,8 @@ namespace Mono.Debugger.Backend
 		{
 			check_disposed ();
 			TargetError error = mono_debugger_server_stop (server_handle);
+			if(error == TargetError.AlreadyStopped)
+				change_target_state (TargetState.Stopped);
 			return error == TargetError.None;
 		}
 
@@ -1252,7 +1296,7 @@ namespace Mono.Debugger.Backend
 
 		public Module[] Modules {
 			get {
-				return new Module[] { bfd.Module };
+				return new Module[] { exe.Module };
 			}
 		}
 
@@ -1321,23 +1365,29 @@ namespace Mono.Debugger.Backend
 		protected string GetApplication (out string cwd, out string[] cmdline_args)
 		{
 			IntPtr data = IntPtr.Zero;
+			IntPtr p_exe = IntPtr.Zero;
+			IntPtr p_cwd = IntPtr.Zero;
 			try {
 				int count;
 				string exe_file;
 				check_error (mono_debugger_server_get_application (
-						     server_handle, out exe_file, out cwd,
+						     server_handle, out p_exe, out p_cwd,
 						     out count, out data));
 
 				cmdline_args = new string [count];
+				exe_file = Marshal.PtrToStringAnsi (p_exe);
+				cwd = Marshal.PtrToStringAnsi (p_cwd);
 
 				for (int i = 0; i < count; i++) {
 					IntPtr ptr = Marshal.ReadIntPtr (data, i * IntPtr.Size);
-					cmdline_args [i] = Marshal.PtrToStringAuto (ptr);
+					cmdline_args [i] = Marshal.PtrToStringAnsi (ptr);
 				}
 
 				return exe_file;
 			} finally {
 				g_free (data);
+				g_free (p_exe);
+				g_free (p_cwd);
 			}
 		}
 
@@ -1601,6 +1651,19 @@ namespace Mono.Debugger.Backend
 			}
 		}
 
+		public bool Has_SIGWINCH {
+			get { return has_signals && (signal_info.SIGWINCH > 0); }
+		}
+
+		public int SIGWINCH {
+			get {
+				if (!has_signals || (signal_info.SIGWINCH < 0))
+					throw new InvalidOperationException ();
+
+				return signal_info.SIGWINCH;
+			}
+		}
+
 		public bool IsManagedSignal (int signal)
 		{
 			if (!has_signals)
@@ -1636,6 +1699,44 @@ namespace Mono.Debugger.Backend
 			}
 		}
 
+		public static bool HasThreadEvents {
+			get {
+				ServerCapabilities capabilities = mono_debugger_server_get_capabilities ();
+				return (capabilities & ServerCapabilities.THREAD_EVENTS) != 0;
+			}
+		}
+
+		//
+		// Whether we can detach from any target.
+		//
+		// Background:
+		//
+		// The Linux kernel allows detaching from any traced child, even if we did not
+		// previously attach to it.
+		//
+
+		public static bool CanDetachAny {
+			get {
+				ServerCapabilities capabilities = mono_debugger_server_get_capabilities ();
+				return (capabilities & ServerCapabilities.CAN_DETACH_ANY) != 0;
+			}
+		}
+
+		public static OperatingSystemBackend CreateOperatingSystemBackend (ProcessServant process)
+		{
+			ServerType type = mono_debugger_server_get_server_type ();
+			switch (type) {
+			case ServerType.LINUX_PTRACE:
+				return new LinuxOperatingSystem (process);
+			case ServerType.DARWIN:
+				return new DarwinOperatingSystem (process);
+			case ServerType.WINDOWS:
+				return new WindowsOperatingSystem (process);
+			default:
+				throw new NotSupportedException (String.Format ("Unknown server type {0}.", type));
+			}
+		}
+
 		//
 		// IDisposable
 		//
