@@ -1,5 +1,6 @@
 using System;
 using System.IO;
+using System.Linq;
 using System.Collections;
 using System.Collections.Generic;
 using ST = System.Threading;
@@ -7,35 +8,40 @@ using System.Runtime.Serialization;
 using System.Runtime.Serialization.Formatters.Binary;
 
 using Mono.Debugger.Backend;
+using Mono.Debugger.Backend.Mono;
 using Mono.Debugger.Architectures;
 using Mono.Debugger.Languages;
+using Mono.Debugger.Languages.Native;
 using Mono.Debugger.Languages.Mono;
 
 namespace Mono.Debugger.Backend
 {
-	internal class ProcessServant : DebuggerMarshalByRefObject
+	internal class ProcessServant : DebuggerMarshalByRefObject, IOperationHost
 	{
 		Process client;
 		TargetInfo target_info;
 		ThreadManager manager;
 		Architecture architecture;
-		BfdContainer bfd_container;
+		OperatingSystemBackend os;
+		NativeLanguage native_language;
 		SymbolTableManager symtab_manager;
 		MonoThreadManager mono_manager;
 		BreakpointManager breakpoint_manager;
+		Dictionary<int,ExceptionCatchPoint> exception_handlers;
 		ProcessStart start;
 		DebuggerSession session;
 		protected MonoLanguageBackend mono_language;
 		protected ThreadServant main_thread;
 		Hashtable thread_hash;
 
+		ProcessServant parent;
+
 		ThreadDB thread_db;
 
 		bool is_attached;
 		bool is_execed;
 		bool is_forked;
 		bool initialized;
-		ST.ManualResetEvent initialized_event;
 		DebuggerMutex thread_lock_mutex;
 		bool has_thread_lock;
 
@@ -50,8 +56,9 @@ namespace Mono.Debugger.Backend
 
 			thread_lock_mutex = new DebuggerMutex ("thread_lock_mutex");
 
+			stopped_event = new ST.ManualResetEvent (false);
+
 			thread_hash = Hashtable.Synchronized (new Hashtable ());
-			initialized_event = new ST.ManualResetEvent (false);
 
 			target_info = Inferior.GetTargetInfo ();
 			if (target_info.TargetAddressSize == 8)
@@ -69,9 +76,12 @@ namespace Mono.Debugger.Backend
 
 			breakpoint_manager = new BreakpointManager ();
 
+			exception_handlers = new Dictionary<int,ExceptionCatchPoint> ();
+
 			symtab_manager = new SymbolTableManager (session);
 
-			bfd_container = new BfdContainer (this);
+			os = Inferior.CreateOperatingSystemBackend (this);
+			native_language = new NativeLanguage (this, os, target_info);
 
 			session.OnProcessCreated (client);
 		}
@@ -84,11 +94,18 @@ namespace Mono.Debugger.Backend
 			this.is_forked = true;
 			this.initialized = true;
 
+			this.parent = parent;
+
 			breakpoint_manager = new BreakpointManager (parent.breakpoint_manager);
 
+			exception_handlers = new Dictionary<int,ExceptionCatchPoint> ();
+			foreach (KeyValuePair<int,ExceptionCatchPoint> catchpoint in parent.exception_handlers)
+				exception_handlers.Add (catchpoint.Key, catchpoint.Value);
+
 			symtab_manager = parent.symtab_manager;
 
-			bfd_container = parent.bfd_container;
+			native_language = parent.native_language;
+			os = parent.os;
 		}
 
 		public int ID {
@@ -100,7 +117,7 @@ namespace Mono.Debugger.Backend
 		}
 
 		public bool CanDetach {
-			get { return is_attached; }
+			get { return is_attached || Inferior.CanDetachAny; }
 		}
 
 		public bool IsExeced {
@@ -123,10 +140,6 @@ namespace Mono.Debugger.Backend
 			get { return architecture; }
 		}
 
-		internal BfdContainer BfdContainer {
-			get { return bfd_container; }
-		}
-
 		internal BreakpointManager BreakpointManager {
 			get { return breakpoint_manager; }
 		}
@@ -137,10 +150,12 @@ namespace Mono.Debugger.Backend
 			}
 		}
 
-		public Language NativeLanguage {
-			get {
-				return bfd_container.NativeLanguage;
-			}
+		public NativeLanguage NativeLanguage {
+			get { return native_language; }
+		}
+
+		public OperatingSystemBackend OperatingSystem {
+			get { return os; }
 		}
 
 		public ThreadServant MainThread {
@@ -159,6 +174,10 @@ namespace Mono.Debugger.Backend
 			get { return mono_manager; }
 		}
 
+		internal bool MonoRuntimeFound {
+			get; private set;
+		}
+
 		public bool IsManaged {
 			get { return mono_manager != null; }
 		}
@@ -175,36 +194,36 @@ namespace Mono.Debugger.Backend
 			get { return start.CommandLineArguments; }
 		}
 
-		internal void ThreadCreated (Inferior inferior, int pid, bool do_attach)
-		{
-			Inferior new_inferior = inferior.CreateThread ();
-			if (do_attach)
-				new_inferior.Attach (pid);
-			else
-				new_inferior.InitializeThread (pid);
+		internal ST.WaitHandle WaitHandle {
+			get { return stopped_event; }
+		}
 
-			SingleSteppingEngine new_thread = new SingleSteppingEngine (
-				manager, this, new_inferior, pid);
+		internal void ThreadCreated (Inferior inferior, int pid, bool do_attach, bool resume_thread)
+		{
+			Inferior new_inferior = inferior.CreateThread (pid, do_attach);
+
+			SingleSteppingEngine new_thread = new SingleSteppingEngine (manager, this, new_inferior, pid);
 
 			Report.Debug (DebugFlags.Threads, "Thread created: {0} {1} {2}", pid, new_thread, do_attach);
 
-			// Order is important: first add the new engine to the manager's hash table,
-			//                     then call inferior.Initialize() / inferior.Attach().
-			manager.AddEngine (new_thread);
-
-			new_thread.StartThread (do_attach, true);
-
-			if ((mono_manager != null) && !do_attach)
+			if (mono_manager != null)
 				mono_manager.ThreadCreated (new_thread);
 
 			if (!do_attach && !is_execed)
 				get_thread_info (inferior, new_thread);
 			OnThreadCreatedEvent (new_thread);
+
+			if (resume_thread) {
+				CommandResult result = current_operation != null ?
+					current_operation : new ThreadCommandResult (new_thread.Thread);
+				new_thread.StartThread (result);
+			}
 		}
 
 		internal void ChildForked (Inferior inferior, int pid)
 		{
 			ProcessServant new_process = new ProcessServant (this, pid);
+			new_process.ProcessStart.StopInMain = false;
 
 			Inferior new_inferior = Inferior.CreateInferior (
 				manager, new_process, new_process.ProcessStart);
@@ -222,26 +241,27 @@ namespace Mono.Debugger.Backend
 
 			new_process.main_thread = new_thread;
 
-			// Order is important: first add the new engine to the manager's hash table,
-			//                     then call inferior.Initialize() / inferior.Attach().
-			manager.AddEngine (new_thread);
-			new_thread.InitAfterFork ();
-
 			manager.Debugger.OnProcessCreatedEvent (new_process);
 			new_process.OnThreadCreatedEvent (new_thread);
+
+			CommandResult result = new_process.CloneParentOperation (new_thread);
+			new_thread.StartForkedChild (result);
 		}
 
-		internal void ChildExecd (Inferior inferior)
+		internal void ChildExecd (SingleSteppingEngine engine, Inferior inferior)
 		{
 			is_execed = true;
 
 			if (!is_forked) {
-				if (bfd_container != null)
-					bfd_container.Dispose ();
-
 				if (mono_language != null)
 					mono_language.Dispose();
 
+				if (native_language != null)
+					native_language.Dispose ();
+
+				if (os != null)
+					os.Dispose ();
+
 				if (symtab_manager != null)
 					symtab_manager.Dispose ();
 			}
@@ -249,17 +269,30 @@ namespace Mono.Debugger.Backend
 			if (breakpoint_manager != null)
 				breakpoint_manager.Dispose ();
 
-			session = session.Clone (start.Options, "@" + id);
-			session.OnProcessCreated (client);
+			session.OnProcessExecd (client);
 
 			breakpoint_manager = new BreakpointManager ();
 
+			exception_handlers = new Dictionary<int,ExceptionCatchPoint> ();
+
 			symtab_manager = new SymbolTableManager (session);
 
-			bfd_container = new BfdContainer (this);
+			os = Inferior.CreateOperatingSystemBackend (this);
+			native_language = new NativeLanguage (this, os, target_info);
 
 			Inferior new_inferior = Inferior.CreateInferior (manager, this, start);
-			new_inferior.InitializeThread (inferior.PID);
+			try {
+				new_inferior.InitializeAfterExec (inferior.PID);
+			} catch (Exception ex) {
+				if ((ex is TargetException) && (((TargetException) ex).Type == TargetError.PermissionDenied)) {
+					Report.Error ("Permission denied when trying to debug exec()ed child {0}, detaching!",
+						      inferior.PID);
+				} else {
+					Report.Error ("InitializeAfterExec() failed on pid {0}: {1}", inferior.PID, ex);
+				}
+				new_inferior.DetachAfterFork ();
+				return;
+			}
 
 			SingleSteppingEngine new_thread = new SingleSteppingEngine (
 				manager, this, new_inferior, inferior.PID);
@@ -277,24 +310,56 @@ namespace Mono.Debugger.Backend
 
 			thread_hash [inferior.PID] = new_thread;
 
-			manager.ProcessExecd (new_thread);
-			new_thread.StartThread (false, false);
+			inferior.Dispose ();
+			inferior = null;
 
 			manager.Debugger.OnProcessExecdEvent (this);
 			manager.Debugger.OnThreadCreatedEvent (new_thread.Thread);
 			initialized = is_forked = false;
+			main_thread = new_thread;
 
-			inferior.Dispose ();
+			if ((engine.Thread.ThreadFlags & Thread.Flags.StopOnExit) != 0)
+				new_thread.Thread.ThreadFlags |= Thread.Flags.StopOnExit;
 
-			Initialize (new_thread, new_inferior, true);
-			new_inferior.Continue ();
+			CommandResult result = engine.OnExecd (new_thread);
+			new_thread.StartExecedChild (result);
+		}
+
+		internal CommandResult StartApplication ()
+		{
+			SingleSteppingEngine engine = new SingleSteppingEngine (manager, this, start);
+
+			initialized = true;
+
+			this.main_thread = engine;
+			engine.Thread.ThreadFlags |= Thread.Flags.StopOnExit;
+
+			if (thread_hash.Contains (engine.PID))
+				thread_hash [engine.PID] = engine;
+			else
+				thread_hash.Add (engine.PID, engine);
+			session.MainThreadGroup.AddThread (engine.Thread.ID);
+
+			session.OnMainProcessCreated (client);
+			manager.Debugger.OnMainProcessCreatedEvent (this);
+
+			CommandResult result = Debugger.StartOperation (start.Session.Config.ThreadingModel, engine);
+			return engine.StartApplication (result);
 		}
 
 		internal void OnProcessExitedEvent ()
 		{
+			DropGlobalThreadLock ();
+
+			if (current_state == ProcessState.Running) {
+				current_state = ProcessState.Exited;
+				current_operation.Completed ();
+				current_operation = null;
+				stopped_event.Set ();
+			}
+
 			if (!is_forked)
 				session.OnProcessExited (client);
-			client.OnProcessExited ();
 			session.MainThreadGroup.RemoveThread (main_thread.ID);
 			manager.Debugger.OnProcessExitedEvent (this);
 		}
@@ -331,129 +396,69 @@ namespace Mono.Debugger.Backend
 			client.OnTargetOutput (is_stderr, output);
 		}
 
-		internal void WaitForApplication ()
+		internal void InitializeMono (Inferior inferior, TargetAddress mdb_debug_info)
 		{
-			initialized_event.WaitOne ();
-		}
+			MonoRuntimeFound = true;
+			mono_manager = MonoThreadManager.Initialize (this, inferior, mdb_debug_info, is_attached);
 
-		internal void Initialize (SingleSteppingEngine engine, Inferior inferior,
-					  bool is_exec)
-		{
-			if (initialized)
+			InitializeThreads (inferior, !is_attached);
+
+			if (mono_manager == null)
 				return;
 
-			if (!is_exec)
-				inferior.InitializeProcess ();
+			mono_manager.InitializeThreads (inferior);
 
-			initialized = true;
-			if (!is_forked || is_exec) {
-				mono_manager = MonoThreadManager.Initialize (
-					manager, inferior, (start.PID != 0) && !is_exec);
-
-				if (is_exec && (mono_manager != null))
-					mono_manager.InitializeAfterExec (inferior);
-
-				if (!is_forked && !is_exec && !is_attached &&
-				    !start.IsNative && (mono_manager == null))
-					throw new TargetException (TargetError.CannotStartTarget,
-								   "Unsupported `mono' executable: {0}",
-								   start.TargetApplication);
-			}
-
-			this.main_thread = engine;
-
-			if (thread_hash.Contains (engine.PID))
-				thread_hash [engine.PID] = engine;
-			else
-				thread_hash.Add (engine.PID, engine);
-			session.MainThreadGroup.AddThread (engine.Thread.ID);
-
-			if (!is_forked && !is_exec) {
-				session.OnMainProcessCreated (client);
-				manager.Debugger.OnMainProcessCreatedEvent (this);
-			}
-
-			if ((start.PID != 0) && !is_exec) {
-				int[] threads = inferior.GetThreads ();
-				foreach (int thread in threads) {
-					if (thread_hash.Contains (thread))
-						continue;
-					ThreadCreated (inferior, thread, true);
-				}
-
-				inferior.InitializeModules ();
-				InitializeThreads (inferior);
-
-				if (mono_manager != null)
-					read_thread_table (inferior);
-			}
-
-			if (!IsManaged)
-				manager.Debugger.Client.OnProcessReachedMainEvent (client);
-
-			initialized_event.Set ();
+			if (is_attached)
+				mono_manager.InitializeAfterAttach (inferior);
 		}
 
-		void read_thread_table (Inferior inferior)
+		internal void InitializeThreads (Inferior inferior, bool resume_threads)
 		{
-			TargetAddress ptr = inferior.ReadAddress (mono_manager.MonoDebuggerInfo.ThreadTable);
-			while (!ptr.IsNull) {
-				int size = 32 + inferior.TargetMemoryInfo.TargetAddressSize;
-				TargetReader reader = new TargetReader (inferior.ReadMemory (ptr, size));
+			if (thread_db != null)
+				return;
 
-				long tid = reader.ReadLongInteger ();
-				TargetAddress lmf_addr = reader.ReadAddress ();
-				TargetAddress end_stack = reader.ReadAddress ();
-
-				TargetAddress extended_notifications_addr = ptr + 24;
-
-				if (inferior.TargetMemoryInfo.TargetAddressSize == 4)
-					tid &= 0x00000000ffffffffL;
-
-				reader.Offset += 8;
-				ptr = reader.ReadAddress ();
-
-				bool found = false;
-				foreach (SingleSteppingEngine engine in thread_hash.Values) {
-					if (engine.TID != tid)
-						continue;
-
-					engine.SetManagedThreadData (lmf_addr, extended_notifications_addr);
-					engine.OnManagedThreadCreated (end_stack);
-					found = true;
-					break;
-				}
-
-				if (!found)
-					Report.Error ("Cannot find thread {0:x} in {1}",
-						      tid, start.CommandLine);
-			}
-		}
-
-		internal void InitializeThreads (Inferior target)
-		{
-			thread_db = ThreadDB.Create (this, target);
+			thread_db = ThreadDB.Create (this, inferior);
 			if (thread_db == null) {
-				if (IsManaged)
-					Report.Error ("Failed to initialize thread_db on {0}",
-						      start.CommandLine);
-				return;
+				if (!IsManaged)
+					return;
+
+				Report.Error ("Failed to initialize thread_db on {0}", start.CommandLine);
+				throw new TargetException (TargetError.CannotStartTarget,
+							   "Failed to initialize thread_db on {0}", start.CommandLine);
 			}
 
-			thread_db.GetThreadInfo (target, delegate (int lwp, long tid) {
+			int[] threads = inferior.GetThreads ();
+			foreach (int thread in threads) {
+				if (thread_hash.Contains (thread))
+					continue;
+				ThreadCreated (inferior, thread, Inferior.HasThreadEvents, resume_threads);
+			}
+
+			thread_db.GetThreadInfo (inferior, delegate (int lwp, long tid) {
 				SingleSteppingEngine engine = (SingleSteppingEngine) thread_hash [lwp];
 				if (engine == null) {
-					Report.Error ("Unknown thread {0} in {1}", lwp,
-						      start.CommandLine);
+					Report.Error ("Unknown thread {0} in {1}", lwp, start.CommandLine);
 					return;
 				}
 				engine.SetTID (tid);
 			});
 		}
 
+		public bool CheckForThreads(ArrayList check_threads)
+		{
+			if(thread_db == null)
+				return false;
+			thread_db.GetThreadInfo (null, delegate (int lwp, long tid) {
+				check_threads.Add(lwp);
+			} );
+			return true;
+		}
+
 		void get_thread_info (Inferior inferior, SingleSteppingEngine engine)
 		{
 			if (thread_db == null) {
+				if (mono_manager == null)
+					return;
 				Report.Error ("Failed to initialize thread_db on {0}: {1} {2}",
 					      start.CommandLine, start, Environment.StackTrace);
 				throw new InternalError ();
@@ -481,9 +486,9 @@ namespace Mono.Debugger.Backend
 			}
 
 			if (thread_db == null) {
-				Report.Error ("Failed to initialize thread_db on {0}: {1} {2}",
-					      start.CommandLine, start, Environment.StackTrace);
-				throw new InternalError ();
+				Report.Error ("Failed to initialize thread_db on {0}: {1}",
+					      start.CommandLine, start);
+				return null;
 			}
 
 			SingleSteppingEngine result = null;
@@ -503,7 +508,16 @@ namespace Mono.Debugger.Backend
 
 		public void Kill ()
 		{
-			main_thread.Kill ();
+			if (!Inferior.HasThreadEvents) {
+				SingleSteppingEngine[] sses = new SingleSteppingEngine [thread_hash.Count];
+				thread_hash.Values.CopyTo (sses, 0);
+				foreach (SingleSteppingEngine sse in sses)
+					sse.SetKilledFlag ();
+				foreach (SingleSteppingEngine sse in sses)
+					sse.Kill ();
+			} else {
+				main_thread.Kill ();
+			}
 		}
 
 		public void Detach ()
@@ -536,13 +550,12 @@ namespace Mono.Debugger.Backend
 		{
 			if (mono_language == null)
 				throw new SymbolTableException (
-						"Cannot load .NET assembly {0} while " +
-						"debugging an unmanaged application",
+						"Cannot load .NET assembly `{0}' while " +
+						"debugging an unmanaged application.",
 						filename);
 
 			if (!mono_language.TryFindImage (thread, filename))
-				bfd_container.AddFile (thread.TargetMemoryInfo, filename,
-						       TargetAddress.Null, true, false);
+				throw new SymbolTableException ("Could not find any .NET assembly `{0}'.", filename);
 		}
 
 		internal MonoLanguageBackend MonoLanguage {
@@ -597,9 +610,20 @@ namespace Mono.Debugger.Backend
 			}
 		}
 
+		internal SingleSteppingEngine[] Engines {
+			get {
+				lock (thread_hash.SyncRoot) {
+					int count = thread_hash.Count;
+					SingleSteppingEngine[] engines = new SingleSteppingEngine [count];
+					thread_hash.Values.CopyTo (engines, 0);
+					return engines;
+				}
+			}
+		}
+
 		public TargetAddress LookupSymbol (string name)
 		{
-			return bfd_container.LookupSymbol (name);
+			return os.LookupSymbol (name);
 		}
 
 		public Thread[] GetThreads ()
@@ -657,17 +681,232 @@ namespace Mono.Debugger.Backend
 			thread_lock_mutex.Unlock ();
 			Report.Debug (DebugFlags.Threads,
 				      "Released global thread lock: {0}", caller);
+		}
 
-			foreach (ThreadServant thread in thread_hash.Values) {
-				if (thread == caller)
+		internal void DropGlobalThreadLock ()
+		{
+			if (thread_hash.Count != 0)
+				throw new InternalError ();
+
+			if (has_thread_lock) {
+				has_thread_lock = false;
+				thread_lock_mutex.Unlock ();
+			}
+		}
+
+#region User Threads
+
+		internal enum ProcessState
+		{
+			SingleThreaded,
+			Running,
+			Stopping,
+			Stopped,
+			Exited
+		}
+
+		protected ST.ManualResetEvent stopped_event;
+
+		ProcessState current_state = ProcessState.Stopped;
+		OperationCommandResult current_operation = null;
+
+		public void OperationCompleted (SingleSteppingEngine caller, TargetEventArgs result, ThreadingModel model)
+		{
+			if (!ThreadManager.InBackgroundThread && Inferior.HasThreadEvents)
+				throw new InternalError ();
+
+			if (current_state == ProcessState.Stopping)
+				return;
+			else if (current_state != ProcessState.Running)
+				throw new InternalError ();
+
+			if ((result != null) && (caller != main_thread) &&
+			    ((result.Type == TargetEventType.TargetExited) || (result.Type == TargetEventType.TargetSignaled)))
+				return;
+
+			current_state = ProcessState.Stopping;
+			SuspendUserThreads (model, caller);
+
+			lock (this) {
+				current_state = ProcessState.Stopped;
+				current_operation.Completed ();
+				current_operation = null;
+				stopped_event.Set ();
+			}
+		}
+
+		protected void SuspendUserThreads (ThreadingModel model, SingleSteppingEngine caller)
+		{
+			Report.Debug (DebugFlags.Threads,
+				      "Suspending user threads: {0} {1}", model, caller);
+
+			foreach (SingleSteppingEngine engine in thread_hash.Values) {
+				Report.Debug (DebugFlags.Threads, "  check user thread: {0} {1}",
+					      engine, engine.Thread.ThreadFlags);
+				if (engine == caller)
 					continue;
-				thread.ReleaseThreadLockDone ();
+				if (((engine.Thread.ThreadFlags & Thread.Flags.Immutable) != 0) &&
+				    ((model & ThreadingModel.StopImmutableThreads) == 0))
+					continue;
+				if (((engine.Thread.ThreadFlags & Thread.Flags.Daemon) != 0) &&
+				    ((model & ThreadingModel.StopDaemonThreads) == 0))
+					continue;
+				engine.SuspendUserThread ();
 			}
 
 			Report.Debug (DebugFlags.Threads,
-				      "Released global thread lock #1: {0}", caller);
+				      "Done suspending user threads: {0} {1}", model, caller);
 		}
 
+		protected void ResumeUserThreads (ThreadingModel model, SingleSteppingEngine caller)
+		{
+			Report.Debug (DebugFlags.Threads,
+				      "Resuming user threads: {0}", caller);
+
+			foreach (SingleSteppingEngine engine in thread_hash.Values) {
+				if (engine == caller)
+					continue;
+				if ((engine.Thread.ThreadFlags & Thread.Flags.AutoRun) == 0)
+					continue;
+				if (((engine.Thread.ThreadFlags & Thread.Flags.Immutable) != 0) &&
+				    ((model & ThreadingModel.StopImmutableThreads) == 0))
+					continue;
+				if (((engine.Thread.ThreadFlags & Thread.Flags.Daemon) != 0) &&
+				    ((model & ThreadingModel.StopDaemonThreads) == 0))
+					continue;
+
+				CommandResult result;
+				if (current_operation != null)
+					result = current_operation;
+				else
+					result = new ThreadCommandResult (engine.Thread);
+
+				engine.ResumeUserThread (result);
+			}
+
+			Report.Debug (DebugFlags.Threads,
+				      "Resumed user threads: {0}", caller);
+		}
+
+		internal CommandResult StartOperation (ThreadingModel model, SingleSteppingEngine caller)
+		{
+			if (!ThreadManager.InBackgroundThread)
+				throw new InternalError ();
+
+			if ((current_state != ProcessState.Stopped) && (current_state != ProcessState.SingleThreaded))
+				throw new TargetException (TargetError.NotStopped);
+
+			if ((model & ThreadingModel.ThreadingMode) == ThreadingModel.Single) {
+				current_state = ProcessState.SingleThreaded;
+				if ((model & ThreadingModel.ResumeThreads) != 0)
+					ResumeUserThreads (model, caller);
+				return new ThreadCommandResult (caller.Thread);
+			} else if ((model & ThreadingModel.ThreadingMode) != ThreadingModel.Process) {
+				throw new ArgumentException ();
+			}
+
+			lock (this) {
+				current_state = ProcessState.Running;
+				stopped_event.Reset ();
+				current_operation = new ProcessCommandResult (this, model);
+			}
+
+			ResumeUserThreads (model, caller);
+			return current_operation;
+		}
+
+		internal void StartGlobalOperation (ThreadingModel model, SingleSteppingEngine caller, OperationCommandResult operation)
+		{
+			if (!ThreadManager.InBackgroundThread)
+				throw new InternalError ();
+
+			if ((current_state != ProcessState.Stopped) && (current_state != ProcessState.SingleThreaded))
+				throw new TargetException (TargetError.NotStopped);
+
+			lock (this) {
+				current_state = ProcessState.Running;
+				stopped_event.Reset ();
+				current_operation = operation;
+			}
+
+			ResumeUserThreads (model, caller);
+		}
+
+		protected CommandResult CloneParentOperation (SingleSteppingEngine new_thread)
+		{
+			if (parent.current_state == ProcessState.SingleThreaded) {
+				current_state = ProcessState.SingleThreaded;
+				return new ThreadCommandResult (new_thread.Thread);
+			}
+
+			if (parent.current_state != ProcessState.Running)
+				throw new InternalError ();
+
+			current_state = ProcessState.Running;
+			if ((parent.current_operation.ThreadingModel & ThreadingModel.ThreadingMode) == ThreadingModel.Global)
+				current_operation = parent.current_operation;
+			else if ((parent.current_operation.ThreadingModel & ThreadingModel.ThreadingMode) == ThreadingModel.Process)
+				current_operation = new ProcessCommandResult (this, parent.current_operation.ThreadingModel);
+			else
+				throw new InternalError ();
+
+			return current_operation;
+		}
+
+		internal void Stop ()
+		{
+			main_thread.Invoke (delegate {
+				current_state = ProcessState.Stopping;
+
+				SuspendUserThreads (ThreadingModel.Process, null);
+				current_state = ProcessState.Stopped;
+				if (current_operation != null) {
+					current_operation.Completed ();
+					current_operation = null;
+				}
+				stopped_event.Set ();
+				return null;
+			}, null);
+		}
+
+		ST.WaitHandle IOperationHost.WaitHandle {
+			get { return stopped_event; }
+		}
+
+		void IOperationHost.SendResult (SingleSteppingEngine sse, TargetEventArgs args)
+		{
+			Debugger.SendTargetEvent (sse, args);
+		}
+
+		void IOperationHost.Abort ()
+		{
+			Stop ();
+		}
+
+		protected class ProcessCommandResult : OperationCommandResult
+		{
+			public ProcessServant Process {
+				get; private set;
+			}
+
+			internal override IOperationHost Host {
+				get { return Process; }
+			}
+
+			internal ProcessCommandResult (ProcessServant process, ThreadingModel model)
+				: base (model)
+			{
+				this.Process = process;
+			}
+
+			internal override void OnExecd (SingleSteppingEngine new_thread)
+			{
+				Process = new_thread.Process;
+			}
+		}
+
+#endregion
+
 		public bool ActivatePendingBreakpoints (CommandResult result)
 		{
 			return ((SingleSteppingEngine) main_thread).ManagedCallback (
@@ -676,23 +915,9 @@ namespace Mono.Debugger.Backend
 				}, result);
 		}
 
-		//
-		// Stopping / resuming all threads for the GUI
-		//
-
-		internal void OnTargetEvent (SingleSteppingEngine sse, TargetEventArgs args)
+		public override string ToString ()
 		{
-			client.OnTargetEvent (sse, args);
-		}
-
-		internal void OnEnterNestedBreakState (SingleSteppingEngine sse)
-		{
-			client.OnEnterNestedBreakState (sse);
-		}
-
-		internal void OnLeaveNestedBreakState (SingleSteppingEngine sse)
-		{
-			client.OnLeaveNestedBreakState (sse);
+			return String.Format ("{0} ({1})", GetType (), ID);
 		}
 
 		//
@@ -715,16 +940,21 @@ namespace Mono.Debugger.Backend
 					architecture = null;
 				}
 
-				if (bfd_container != null) {
-					bfd_container.Dispose ();
-					bfd_container = null;
-				}
-
 				if (mono_language != null) {
 					mono_language.Dispose();
 					mono_language = null;
 				}
 
+				if (native_language != null) {
+					native_language.Dispose ();
+					native_language = null;
+				}
+
+				if (os != null) {
+					os.Dispose ();
+					os = null;
+				}
+
 				if (symtab_manager != null) {
 					symtab_manager.Dispose ();
 					symtab_manager = null;
@@ -746,6 +976,8 @@ namespace Mono.Debugger.Backend
 				thread_lock_mutex = null;
 			}
 
+			exception_handlers = null;
+
 			manager.RemoveProcess (this);
 		}
 
