@@ -110,13 +110,6 @@ namespace Mono.Debugger.Backend
 		SingleSteppingEngine current_event = null;
 		int current_event_status = 0;
 
-		public ProcessServant StartApplication (ProcessStart start)
-		{
-			ProcessServant process = CreateProcess (start);
-			process.WaitForApplication ();
-			return process;
-		}
-
 		public ProcessServant OpenCoreFile (ProcessStart start, out Thread[] threads)
 		{
 			CoreFile core = CoreFile.OpenCoreFile (this, start);
@@ -130,17 +123,6 @@ namespace Mono.Debugger.Backend
 			engine_hash.Add (engine.ID, engine);
 		}
 
-		internal void ProcessExecd (SingleSteppingEngine engine)
-		{
-			SingleSteppingEngine old_engine = (SingleSteppingEngine) thread_hash [engine.PID];
-			if (old_engine != null) {
-				thread_hash [engine.PID] = engine;
-				engine_hash.Remove (old_engine.ID);
-			} else
-				thread_hash.Add (engine.PID, engine);
-			engine_hash.Add (engine.ID, engine);
-		}
-
 		internal void RemoveProcess (ProcessServant process)
 		{
 			processes.Remove (process);
@@ -168,13 +150,11 @@ namespace Mono.Debugger.Backend
 				return true;
 			}
 
-			inferior.Process.Initialize (engine, inferior, false);
-
 			if (cevent.Type == Inferior.ChildEventType.CHILD_CREATED_THREAD) {
 				int pid = (int) cevent.Argument;
+				inferior.Process.ThreadCreated (inferior, pid, false, true);
 				if (pending_sigstops.ContainsKey (pid))
 					pending_sigstops.Remove (pid);
-				inferior.Process.ThreadCreated (inferior, pid, false);
 				resume_target = true;
 				return true;
 			}
@@ -186,7 +166,9 @@ namespace Mono.Debugger.Backend
 			}
 
 			if (cevent.Type == Inferior.ChildEventType.CHILD_EXECD) {
-				inferior.Process.ChildExecd (inferior);
+				thread_hash.Remove (engine.PID);
+				engine_hash.Remove (engine.ID);
+				inferior.Process.ChildExecd (engine, inferior);
 				resume_target = false;
 				return true;
 			}
@@ -197,6 +179,9 @@ namespace Mono.Debugger.Backend
 						Inferior.ChildEventType.CHILD_STOPPED, 0, 0, 0);
 					resume_target = true;
 					return true;
+				} else if (inferior.Has_SIGWINCH && (cevent.Argument == inferior.SIGWINCH)) {
+					resume_target = true;
+					return true;
 				} else if (inferior.HasSignals && (cevent.Argument == inferior.Kernel_SIGRTMIN+1)) {
 					// __SIGRTMIN and __SIGRTMIN+1 are used internally by the threading library
 					resume_target = true;
@@ -204,6 +189,11 @@ namespace Mono.Debugger.Backend
 				}
 			}
 
+			if (inferior.Process.OperatingSystem.CheckForPendingMonoInit (inferior)) {
+				resume_target = true;
+				return true;
+			}
+
 			bool retval = false;
 			resume_target = false;
 			if (inferior.Process.MonoManager != null)
@@ -211,7 +201,7 @@ namespace Mono.Debugger.Backend
 					engine, inferior, ref cevent, out resume_target);
 
 			if ((cevent.Type == Inferior.ChildEventType.CHILD_EXITED) ||
-			     (cevent.Type == Inferior.ChildEventType.CHILD_SIGNALED)) {
+			    (cevent.Type == Inferior.ChildEventType.CHILD_SIGNALED)) {
 				thread_hash.Remove (engine.PID);
 				engine_hash.Remove (engine.ID);
 				engine.OnThreadExited (cevent);
@@ -222,6 +212,15 @@ namespace Mono.Debugger.Backend
 			return retval;
 		}
 
+		internal bool HasPendingSigstopForNewThread (int pid)
+		{
+			if (!pending_sigstops.ContainsKey (pid))
+				return false;
+
+			pending_sigstops.Remove (pid);
+			return true;
+		}
+
 		public DebuggerServant Debugger {
 			get { return backend; }
 		}
@@ -258,7 +257,7 @@ namespace Mono.Debugger.Backend
 				return command.Result;
 		}
 
-		internal ProcessServant CreateProcess (ProcessStart start)
+		public ProcessServant StartApplication (ProcessStart start, out CommandResult result)
 		{
 			Command command = new Command (CommandType.CreateProcess, start);
 
@@ -277,8 +276,11 @@ namespace Mono.Debugger.Backend
 
 			if (command.Result is Exception)
 				throw (Exception) command.Result;
-			else
-				return (ProcessServant) command.Result;
+			else {
+				var pair = (KeyValuePair<CommandResult,ProcessServant>) command.Result;
+				result = pair.Key;
+				return pair.Value;
+			}
 		}
 
 		internal void AddPendingEvent (SingleSteppingEngine engine, Inferior.ChildEvent cevent)
@@ -356,8 +358,9 @@ namespace Mono.Debugger.Backend
 			// until we finished the command and sent the result.
 			if (command.Type == CommandType.TargetAccess) {
 				try {
-					command.Result = command.Engine.Invoke (
-						(TargetAccessDelegate) command.Data1, command.Data2);
+					if(command.Engine.Inferior != null)
+						command.Result = command.Engine.Invoke (
+							(TargetAccessDelegate) command.Data1, command.Data2);
 				} catch (ST.ThreadAbortException) {
 					return;
 				} catch (Exception ex) {
@@ -371,18 +374,13 @@ namespace Mono.Debugger.Backend
 				try {
 					ProcessStart start = (ProcessStart) command.Data1;
 					ProcessServant process = new ProcessServant (this, start);
-
-					CommandResult result;
-					SingleSteppingEngine sse = new SingleSteppingEngine (
-						this, process, start, out result);
-
-					thread_hash.Add (sse.PID, sse);
-					engine_hash.Add (sse.ID, sse);
 					processes.Add (process);
 
+					CommandResult result = process.StartApplication ();
+
 					RequestWait ();
 
-					command.Result = process;
+					command.Result = new KeyValuePair<CommandResult,ProcessServant> (result, process);
 				} catch (ST.ThreadAbortException) {
 					return;
 				} catch (Exception ex) {
@@ -514,10 +512,64 @@ namespace Mono.Debugger.Backend
 			if (abort_requested || (pid <= 0))
 				return true;
 
-			SingleSteppingEngine event_engine = (SingleSteppingEngine) thread_hash [pid];
-			if (event_engine == null) {
+			if(!Inferior.HasThreadEvents)
+			{
 				int arg;
 				Inferior.ChildEventType etype = mono_debugger_server_dispatch_simple (status, out arg);
+				SingleSteppingEngine engine = (SingleSteppingEngine) thread_hash [pid];
+				if(etype == Inferior.ChildEventType.CHILD_EXITED) {
+					if(engine != null) {
+						SingleSteppingEngine[] sses = new SingleSteppingEngine [thread_hash.Count];
+						thread_hash.Values.CopyTo (sses, 0);
+						foreach(SingleSteppingEngine sse in sses)
+							sse.ProcessEvent (status);
+						Dispose();
+						waiting = false;
+						return true;
+					}
+					else
+						goto again;
+				}
+
+				if (engine == null) {
+					SingleSteppingEngine[] sses = new SingleSteppingEngine [thread_hash.Count];
+					thread_hash.Values.CopyTo (sses, 0);				
+					Inferior inferior = sses[0].Inferior;		
+					inferior.Process.ThreadCreated (inferior, pid, false, true);
+					goto again;
+				}
+
+				ArrayList check_threads = new ArrayList();
+				bool got_threads = true;
+				foreach(ProcessServant process in processes)
+					got_threads = got_threads && process.CheckForThreads(check_threads);
+
+				if(got_threads) {
+					int[] lwps = new int [thread_hash.Count];
+					thread_hash.Keys.CopyTo (lwps, 0);
+					foreach(int lwp in lwps) {
+						if(!check_threads.Contains(lwp)) {
+							SingleSteppingEngine old_engine = (SingleSteppingEngine) thread_hash [lwp];					
+							thread_hash.Remove (old_engine.PID);
+							engine_hash.Remove (old_engine.ID);
+							old_engine.ProcessServant.OnThreadExitedEvent (old_engine);
+							old_engine.Dispose ();
+						}
+					}
+				}
+			}
+
+			SingleSteppingEngine event_engine = (SingleSteppingEngine) thread_hash [pid];
+			if (event_engine == null && Inferior.HasThreadEvents) {
+				int arg;
+				Inferior.ChildEventType etype = mono_debugger_server_dispatch_simple (status, out arg);
+
+				/*
+				 * Ignore exit events from unknown children.
+				 */
+
+				if ((etype == Inferior.ChildEventType.CHILD_EXITED) && (arg == 0))
+					goto again;
 
 				/*
 				 * There is a race condition in the Linux kernel which shows up on >= 2.6.27:
