@@ -10,7 +10,6 @@
 #include <fcntl.h>
 #include <errno.h>
 
-#include "x86-linux-ptrace.h"
 #include "x86-arch.h"
 
 typedef struct
@@ -29,6 +28,7 @@ struct ArchInfo
 	GPtrArray *callback_stack;
 	CodeBufferData *code_buffer;
 	guint64 dr_control, dr_status;
+	BreakpointManager *hw_bpm;
 	int dr_regs [DR_NADDR];
 };
 
@@ -55,6 +55,7 @@ x86_arch_initialize (void)
 	ArchInfo *arch = g_new0 (ArchInfo, 1);
 
 	arch->callback_stack = g_ptr_array_new ();
+	arch->hw_bpm = mono_debugger_breakpoint_manager_new ();
 
 	return arch;
 }
@@ -63,6 +64,7 @@ void
 x86_arch_finalize (ArchInfo *arch)
 {
 	g_ptr_array_free (arch->callback_stack, TRUE);
+	mono_debugger_breakpoint_manager_free (arch->hw_bpm);
 	g_free (arch);
 }
 
@@ -70,7 +72,8 @@ static ServerCommandError
 server_ptrace_current_insn_is_bpt (ServerHandle *handle, guint32 *is_breakpoint)
 {
 	mono_debugger_breakpoint_manager_lock ();
-	if (mono_debugger_breakpoint_manager_lookup (handle->bpm, INFERIOR_REG_EIP (handle->arch->current_regs)))
+	if (mono_debugger_breakpoint_manager_lookup (handle->arch->hw_bpm, INFERIOR_REG_EIP (handle->arch->current_regs)) ||
+	    mono_debugger_breakpoint_manager_lookup (handle->bpm, INFERIOR_REG_EIP (handle->arch->current_regs)))
 		*is_breakpoint = TRUE;
 	else
 		*is_breakpoint = FALSE;
@@ -166,6 +169,10 @@ server_ptrace_call_method (ServerHandle *handle, guint64 method_address,
 	if (result != COMMAND_ERROR_NONE)
 		return result;
 
+	result = _server_ptrace_make_memory_executable (handle, (guint32) new_esp, size);
+	if (result != COMMAND_ERROR_NONE)
+		return result;
+
 	INFERIOR_REG_ORIG_EAX (arch->current_regs) = -1;
 	INFERIOR_REG_ESP (arch->current_regs) = INFERIOR_REG_EIP (arch->current_regs) = new_esp;
 
@@ -233,6 +240,10 @@ server_ptrace_call_method_1 (ServerHandle *handle, guint64 method_address,
 	if (result != COMMAND_ERROR_NONE)
 		return result;
 
+	result = _server_ptrace_make_memory_executable (handle, (guint32) new_esp, size);
+	if (result != COMMAND_ERROR_NONE)
+		return result;
+
 	INFERIOR_REG_ORIG_EAX (arch->current_regs) = -1;
 	INFERIOR_REG_ESP (arch->current_regs) = INFERIOR_REG_EIP (arch->current_regs) = new_esp;
 
@@ -296,6 +307,10 @@ server_ptrace_call_method_2 (ServerHandle *handle, guint64 method_address,
 	if (result != COMMAND_ERROR_NONE)
 		return result;
 
+	result = _server_ptrace_make_memory_executable (handle, (guint32) new_esp, size);
+	if (result != COMMAND_ERROR_NONE)
+		return result;
+
 	INFERIOR_REG_ORIG_EAX (arch->current_regs) = -1;
 	INFERIOR_REG_EIP (arch->current_regs) = method_address;
 	INFERIOR_REG_ESP (arch->current_regs) = new_esp;
@@ -358,6 +373,10 @@ server_ptrace_call_method_3 (ServerHandle *handle, guint64 method_address,
 	if (result != COMMAND_ERROR_NONE)
 		return result;
 
+	result = _server_ptrace_make_memory_executable (handle, (guint32) new_esp, size);
+	if (result != COMMAND_ERROR_NONE)
+		return result;
+
 	INFERIOR_REG_ORIG_EAX (arch->current_regs) = -1;
 	INFERIOR_REG_EIP (arch->current_regs) = method_address;
 	INFERIOR_REG_ESP (arch->current_regs) = new_esp;
@@ -432,6 +451,10 @@ server_ptrace_call_method_invoke (ServerHandle *handle, guint64 invoke_method,
 	if (result != COMMAND_ERROR_NONE)
 		return result;
 
+	result = _server_ptrace_make_memory_executable (handle, (guint32) new_esp, size);
+	if (result != COMMAND_ERROR_NONE)
+		return result;
+
 	INFERIOR_REG_ORIG_EAX (arch->current_regs) = -1;
 	INFERIOR_REG_EIP (arch->current_regs) = invoke_method;
 	INFERIOR_REG_ESP (arch->current_regs) = new_esp;
@@ -463,6 +486,35 @@ check_breakpoint (ServerHandle *handle, guint64 address, guint64 *retval)
 	return TRUE;
 }
 
+static BreakpointInfo *
+lookup_breakpoint (ServerHandle *handle, guint32 idx, BreakpointManager **out_bpm)
+{
+	BreakpointInfo *info;
+
+	mono_debugger_breakpoint_manager_lock ();
+	info = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->arch->hw_bpm, idx);
+	if (info) {
+		if (out_bpm)
+			*out_bpm = handle->arch->hw_bpm;
+		mono_debugger_breakpoint_manager_unlock ();
+		return info;
+	}
+
+	info = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, idx);
+	if (info) {
+		if (out_bpm)
+			*out_bpm = handle->bpm;
+		mono_debugger_breakpoint_manager_unlock ();
+		return info;
+	}
+
+	if (out_bpm)
+		*out_bpm = NULL;
+
+	mono_debugger_breakpoint_manager_unlock ();
+	return info;
+}
+
 static CallbackData *
 get_callback_data (ArchInfo *arch)
 {
@@ -948,15 +1000,6 @@ server_ptrace_insert_breakpoint (ServerHandle *handle, guint64 address, guint32
 	mono_debugger_breakpoint_manager_lock ();
 	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup (handle->bpm, address);
 	if (breakpoint) {
-		/*
-		 * You cannot have a hardware breakpoint and a normal breakpoint on the same
-		 * instruction.
-		 */
-		if (breakpoint->is_hardware_bpt) {
-			mono_debugger_breakpoint_manager_unlock ();
-			return COMMAND_ERROR_DR_OCCUPIED;
-		}
-
 		breakpoint->refcount++;
 		goto done;
 	}
@@ -986,13 +1029,14 @@ server_ptrace_insert_breakpoint (ServerHandle *handle, guint64 address, guint32
 }
 
 static ServerCommandError
-server_ptrace_remove_breakpoint (ServerHandle *handle, guint32 bhandle)
+server_ptrace_remove_breakpoint (ServerHandle *handle, guint32 idx)
 {
+	BreakpointManager *bpm;
 	BreakpointInfo *breakpoint;
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, bhandle);
+	breakpoint = lookup_breakpoint (handle, idx, &bpm);
 	if (!breakpoint) {
 		result = COMMAND_ERROR_NO_SUCH_BREAKPOINT;
 		goto out;
@@ -1008,7 +1052,7 @@ server_ptrace_remove_breakpoint (ServerHandle *handle, guint32 bhandle)
 		goto out;
 
 	breakpoint->enabled = FALSE;
-	mono_debugger_breakpoint_manager_remove (handle->bpm, (BreakpointInfo *) breakpoint);
+	mono_debugger_breakpoint_manager_remove (bpm, breakpoint);
 
  out:
 	mono_debugger_breakpoint_manager_unlock ();
@@ -1053,11 +1097,6 @@ server_ptrace_insert_hw_breakpoint (ServerHandle *handle, guint32 type, guint32
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup (handle->bpm, address);
-	if (breakpoint) {
-		breakpoint->refcount++;
-		goto done;
-	}
 
 	result = find_free_hw_register (handle, idx);
 	if (result != COMMAND_ERROR_NONE) {
@@ -1081,7 +1120,8 @@ server_ptrace_insert_hw_breakpoint (ServerHandle *handle, guint32 type, guint32
 	}
 
 	breakpoint->enabled = TRUE;
-	mono_debugger_breakpoint_manager_insert (handle->bpm, (BreakpointInfo *) breakpoint);
+	mono_debugger_breakpoint_manager_insert (handle->arch->hw_bpm, (BreakpointInfo *) breakpoint);
+
  done:
 	*bhandle = breakpoint->id;
 	mono_debugger_breakpoint_manager_unlock ();
@@ -1090,13 +1130,13 @@ server_ptrace_insert_hw_breakpoint (ServerHandle *handle, guint32 type, guint32
 }
 
 static ServerCommandError
-server_ptrace_enable_breakpoint (ServerHandle *handle, guint32 bhandle)
+server_ptrace_enable_breakpoint (ServerHandle *handle, guint32 idx)
 {
 	BreakpointInfo *breakpoint;
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, bhandle);
+	breakpoint = lookup_breakpoint (handle, idx, NULL);
 	if (!breakpoint) {
 		mono_debugger_breakpoint_manager_unlock ();
 		return COMMAND_ERROR_NO_SUCH_BREAKPOINT;
@@ -1109,13 +1149,13 @@ server_ptrace_enable_breakpoint (ServerHandle *handle, guint32 bhandle)
 }
 
 static ServerCommandError
-server_ptrace_disable_breakpoint (ServerHandle *handle, guint32 bhandle)
+server_ptrace_disable_breakpoint (ServerHandle *handle, guint32 idx)
 {
 	BreakpointInfo *breakpoint;
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, bhandle);
+	breakpoint = lookup_breakpoint (handle, idx, NULL);
 	if (!breakpoint) {
 		mono_debugger_breakpoint_manager_unlock ();
 		return COMMAND_ERROR_NO_SUCH_BREAKPOINT;
@@ -1311,3 +1351,16 @@ server_ptrace_get_callback_frame (ServerHandle *handle, guint64 stack_pointer,
 
 	return COMMAND_ERROR_NO_CALLBACK_FRAME;
 }
+
+static ServerCommandError
+server_ptrace_restart_notification (ServerHandle *handle)
+{
+	ServerCommandError result;
+
+	if (!handle->mono_runtime ||
+	    (INFERIOR_REG_EIP (handle->arch->current_regs) - 1 != handle->mono_runtime->notification_address))
+		return COMMAND_ERROR_INTERNAL_ERROR;
+
+	INFERIOR_REG_EIP (handle->arch->current_regs)--;
+	return _server_ptrace_set_registers (handle->inferior, &handle->arch->current_regs);
+}
