@@ -36,6 +36,7 @@ typedef enum {
 	COMMAND_ERROR_NOT_IMPLEMENTED,
 	COMMAND_ERROR_IO_ERROR,
 	COMMAND_ERROR_NO_CALLBACK_FRAME,
+	COMMAND_ERROR_PERMISSION_DENIED
 } ServerCommandError;
 
 typedef enum {
@@ -98,6 +99,19 @@ typedef struct
 	guint32 executable_code_last_slot;
 } MonoRuntimeInfo;
 
+typedef enum {
+	SERVER_CAPABILITIES_NONE		= 0,
+	SERVER_CAPABILITIES_THREAD_EVENTS	= 1,
+	SERVER_CAPABILITIES_CAN_DETACH_ANY	= 2
+} ServerCapabilities;
+
+typedef enum {
+	SERVER_TYPE_UNKNOWN			= 0,
+	SERVER_TYPE_LINUX_PTRACE		= 1,
+	SERVER_TYPE_DARWIN			= 2,
+	SERVER_TYPE_WIN32			= 3
+} ServerType;
+
 /* This is an opaque data structure which the backend may use to store stuff. */
 typedef struct InferiorVTable InferiorVTable;
 typedef struct InferiorHandle InferiorHandle;
@@ -120,6 +134,7 @@ typedef struct {
 	int sigsegv;
 	int sigill;
 	int sigbus;
+	int sigwinch;
 	int kernel_sigrtmin;
 	int mono_thread_abort;
 } SignalInfo;
@@ -143,14 +158,22 @@ struct ServerHandle {
 };
 
 struct InferiorVTable {
+	
+	void                  (* static_init)         (void);
+
 	void                  (* global_init)         (void);
 
+	ServerType            (* get_server_type)     (void);
+
+	ServerCapabilities    (* get_capabilities)    (void);
+
 	ServerHandle *        (* create_inferior)     (BreakpointManager  *bpm);
 
 	ServerCommandError    (* initialize_process)  (ServerHandle       *handle);
 
 	ServerCommandError    (* initialize_thread)   (ServerHandle       *handle,
-						       guint32             pid);
+						       guint32             pid,
+						       gboolean            wait);
 
 	void                  (* set_runtime_info)    (ServerHandle       *handle,
 						       MonoRuntimeInfo    *mono_runtime_info);
@@ -415,8 +438,24 @@ struct InferiorVTable {
 						       gboolean           exact_match,
 						       CallbackInfo      *info);
 
+	ServerCommandError    (* restart_notification) (ServerHandle      *handle);
+
 	void         (* get_registers_from_core_file) (guint64           *values,
 						       const guint8      *buffer);
+
+	guint32               (*get_current_pid) (void);
+
+	guint64               (*get_current_thread) (void);
+	
+	void                  (*sem_init) (void);
+
+	void                  (*sem_wait) (void);
+
+	void                  (*sem_post) (void);
+
+	int                   (*sem_get_value) (void);
+
+	int                   (*get_pending_sigint) (void);
 };
 
 /*
@@ -432,6 +471,12 @@ mono_debugger_server_static_init          (void);
 void
 mono_debugger_server_global_init          (void);
 
+ServerCapabilities
+mono_debugger_server_get_capabilities     (void);
+
+ServerType
+mono_debugger_server_get_server_type      (void);
+
 ServerHandle *
 mono_debugger_server_create_inferior      (BreakpointManager  *bpm);
 
@@ -440,7 +485,8 @@ mono_debugger_server_initialize_process   (ServerHandle       *handle);
 
 ServerCommandError
 mono_debugger_server_initialize_thread    (ServerHandle       *handle,
-					   guint32             pid);
+					   guint32             pid,
+					   gboolean            wait);
 
 void
 mono_debugger_server_io_thread_main       (IOThreadData       *io_data,
@@ -662,6 +708,9 @@ mono_debugger_server_get_callback_frame  (ServerHandle        *handle,
 					  gboolean             exact_match,
 					  CallbackInfo        *info);
 
+ServerCommandError
+mono_debugger_server_restart_notification (ServerHandle        *handle);
+
 MonoRuntimeInfo *
 mono_debugger_server_initialize_mono_runtime (guint32 address_size,
 					      guint64 notification_address,
@@ -683,6 +732,13 @@ void
 mono_debugger_server_get_registers_from_core_file (guint64 *values,
 						   const guint8 *buffer);
 
+guint32
+mono_debugger_server_get_current_pid (void);
+
+guint64
+mono_debugger_server_get_current_thread (void);
+
+
 /* POSIX semaphores */
 
 void mono_debugger_server_sem_init (void);
@@ -690,9 +746,6 @@ void mono_debugger_server_sem_wait (void);
 void mono_debugger_server_sem_post (void);
 int mono_debugger_server_sem_get_value (void);
 
-guint32 mono_debugger_server_get_current_pid (void);
-guint64 mono_debugger_server_get_current_thread (void);
-
 int mono_debugger_server_get_pending_sigint (void);
 
 G_END_DECLS
