@@ -12,7 +12,8 @@ typedef enum {
 	STOP_ACTION_CALLBACK,
 	STOP_ACTION_CALLBACK_COMPLETED,
 	STOP_ACTION_NOTIFICATION,
-	STOP_ACTION_RTI_DONE
+	STOP_ACTION_RTI_DONE,
+	STOP_ACTION_INTERNAL_ERROR
 } ChildStoppedAction;
 
 typedef enum {
