@@ -2,6 +2,7 @@
 #define _GNU_SOURCE
 #endif
 
+#include <config.h>
 #include <server.h>
 #include <breakpoints.h>
 #include <stdio.h>
@@ -42,6 +43,10 @@
 #include "x86-freebsd-ptrace.h"
 #endif
 
+#ifdef __MACH__
+#include "darwin-ptrace.h"
+#endif
+
 #include "x86-arch.h"
 
 struct IOThreadData
@@ -49,19 +54,6 @@ struct IOThreadData
 	int output_fd, error_fd;
 };
 
-struct InferiorHandle
-{
-	guint32 pid;
-#ifdef __linux__
-	int mem_fd;
-#endif
-	int stepping;
-	int last_signal;
-	int redirect_fds;
-	int output_fd [2], error_fd [2];
-	int is_thread, is_initialized;
-};
-
 MonoRuntimeInfo *
 mono_debugger_server_initialize_mono_runtime (guint32 address_size,
 					      guint64 notification_address,
@@ -118,32 +110,8 @@ server_ptrace_finalize (ServerHandle *handle)
 	g_free (handle);
 }
 
-static ServerCommandError
-server_ptrace_continue (ServerHandle *handle)
-{
-	InferiorHandle *inferior = handle->inferior;
-
-	errno = 0;
-	inferior->stepping = FALSE;
-	if (ptrace (PT_CONTINUE, inferior->pid, (caddr_t) 1, inferior->last_signal)) {
-		return _server_ptrace_check_errno (inferior);
-	}
-
-	return COMMAND_ERROR_NONE;
-}
-
-static ServerCommandError
-server_ptrace_step (ServerHandle *handle)
-{
-	InferiorHandle *inferior = handle->inferior;
-
-	errno = 0;
-	inferior->stepping = TRUE;
-	if (ptrace (PT_STEP, inferior->pid, (caddr_t) 1, inferior->last_signal))
-		return _server_ptrace_check_errno (inferior);
-
-	return COMMAND_ERROR_NONE;
-}
+#ifndef __MACH__
+#endif
 
 static ServerCommandError
 server_ptrace_resume (ServerHandle *handle)
@@ -168,69 +136,18 @@ server_ptrace_detach (ServerHandle *handle)
 	return COMMAND_ERROR_NONE;
 }
 
-static ServerCommandError
-server_ptrace_kill (ServerHandle *handle)
-{
-	if (ptrace (PTRACE_KILL, handle->inferior->pid, NULL, 0))
-		return COMMAND_ERROR_UNKNOWN_ERROR;
-
-	return COMMAND_ERROR_NONE;
-}
-
 static ServerCommandError
 server_ptrace_peek_word (ServerHandle *handle, guint64 start, guint64 *retval)
 {
 	return server_ptrace_read_memory (handle, start, sizeof (gsize), retval);
 }
 
-static ServerCommandError
-server_ptrace_write_memory (ServerHandle *handle, guint64 start,
-			    guint32 size, gconstpointer buffer)
-{
-	InferiorHandle *inferior = handle->inferior;
-	ServerCommandError result;
-	const long *ptr = buffer;
-	guint64 addr = start;
-	char temp [8];
-
-	while (size >= sizeof (long)) {
-		long word = *ptr++;
-
-		errno = 0;
-		if (ptrace (PT_WRITE_D, inferior->pid, GSIZE_TO_POINTER (addr), word) != 0)
-			return _server_ptrace_check_errno (inferior);
-
-		addr += sizeof (long);
-		size -= sizeof (long);
-	}
-
-	if (!size)
-		return COMMAND_ERROR_NONE;
-
-	result = _server_ptrace_read_memory (handle, addr, sizeof (long), &temp);
-	if (result != COMMAND_ERROR_NONE)
-		return result;
-
-	memcpy (&temp, ptr, size);
-
-	return server_ptrace_write_memory (handle, addr, sizeof (long), &temp);
-}
-
-static ServerCommandError
-server_ptrace_poke_word (ServerHandle *handle, guint64 addr, gsize value)
-{
-	errno = 0;
-	if (ptrace (PT_WRITE_D, handle->inferior->pid, GSIZE_TO_POINTER (addr), value) != 0)
-		return _server_ptrace_check_errno (handle->inferior);
-
-	return COMMAND_ERROR_NONE;
-}
-
 static ServerStatusMessageType
 server_ptrace_dispatch_event (ServerHandle *handle, guint32 status, guint64 *arg,
 			      guint64 *data1, guint64 *data2, guint32 *opt_data_size,
 			      gpointer *opt_data)
 {
+	#ifdef PTRACE_EVENT_CLONE
 	if (status >> 16) {
 		switch (status >> 16) {
 		case PTRACE_EVENT_CLONE: {
@@ -281,8 +198,12 @@ server_ptrace_dispatch_event (ServerHandle *handle, guint32 status, guint64 *arg
 			return MESSAGE_UNKNOWN_ERROR;
 		}
 	}
+	#endif
 
 	if (WIFSTOPPED (status)) {
+#if __MACH__
+		handle->inferior->os.wants_to_run = FALSE;
+#endif
 		guint64 callback_arg, retval, retval2;
 		ChildStoppedAction action;
 		int stopsig;
@@ -291,13 +212,6 @@ server_ptrace_dispatch_event (ServerHandle *handle, guint32 status, guint64 *arg
 		if (stopsig == SIGCONT)
 			stopsig = 0;
 
-		if (!handle->inferior->is_initialized) {
-			x86_arch_remove_hardware_breakpoints (handle);
-			handle->inferior->is_initialized = TRUE;
-			if (stopsig == SIGSTOP)
-				stopsig = 0;
-		}
-
 		action = x86_arch_child_stopped (
 			handle, stopsig, &callback_arg, &retval, &retval2, opt_data_size, opt_data);
 
@@ -405,6 +319,12 @@ server_ptrace_create_inferior (BreakpointManager *bpm)
 {
 	ServerHandle *handle = g_new0 (ServerHandle, 1);
 
+	if ((getuid () == 0) || (geteuid () == 0)) {
+		g_message ("WARNING: Running mdb as root may be a problem because setuid() and\n"
+			   "seteuid() do nothing.\n"
+			   "See http://primates.ximian.com/~martin/blog/entry_150.html for details.");
+	}
+
 	handle->bpm = bpm;
 	handle->inferior = g_new0 (InferiorHandle, 1);
 	handle->arch = x86_arch_initialize ();
@@ -512,6 +432,11 @@ server_ptrace_spawn (ServerHandle *handle, const gchar *working_directory,
 
 	inferior->pid = *child_pid;
 
+#ifndef __MACH__
+	if (!_server_ptrace_wait_for_new_thread (handle))
+		return COMMAND_ERROR_INTERNAL_ERROR;
+#endif
+
 	result = _server_ptrace_setup_inferior (handle);
 	if (result != COMMAND_ERROR_NONE) {
 		if (redirect_fds) {
@@ -521,19 +446,28 @@ server_ptrace_spawn (ServerHandle *handle, const gchar *working_directory,
 		return result;
 	}
 
+#ifdef __MACH__
+	*child_pid = COMPOSED_PID(inferior->pid, inferior->os.thread_index);
+#endif
+
 	return COMMAND_ERROR_NONE;
 }
 
 static ServerCommandError
-server_ptrace_initialize_thread (ServerHandle *handle, guint32 pid)
+server_ptrace_initialize_thread (ServerHandle *handle, guint32 pid, gboolean wait)
 {
 	InferiorHandle *inferior = handle->inferior;
 
-	inferior->pid = pid;
 	inferior->is_thread = TRUE;
 
-	if (!_server_ptrace_wait_for_new_thread (handle))
+#ifdef __MACH__
+	inferior->pid = GET_PID(pid);
+	inferior->os.thread = get_thread_from_index(GET_THREAD_INDEX(pid));
+#else
+	inferior->pid = pid;
+	if (wait && !_server_ptrace_wait_for_new_thread (handle))
 		return COMMAND_ERROR_INTERNAL_ERROR;
+#endif
 
 	return _server_ptrace_setup_inferior (handle);
 }
@@ -552,6 +486,11 @@ server_ptrace_attach (ServerHandle *handle, guint32 pid)
 	inferior->pid = pid;
 	inferior->is_thread = TRUE;
 
+#ifndef __MACH__
+	if (!_server_ptrace_wait_for_new_thread (handle))
+		return COMMAND_ERROR_INTERNAL_ERROR;
+#endif
+
 	return _server_ptrace_setup_inferior (handle);
 }
 
@@ -626,6 +565,54 @@ server_ptrace_set_runtime_info (ServerHandle *handle, MonoRuntimeInfo *mono_runt
 	handle->mono_runtime = mono_runtime;
 }
 
+static guint32
+server_ptrace_get_current_pid (void)
+{
+	return getpid ();
+}
+
+static guint64
+server_ptrace_get_current_thread (void)
+{
+	return pthread_self ();
+}
+
+static gboolean initialized = FALSE;
+int pending_sigint = 0;
+
+static void
+sigint_signal_handler (int _dummy)
+{
+	pending_sigint++;
+	server_ptrace_sem_post ();
+}
+
+static void
+server_ptrace_static_init (void)
+{
+	struct sigaction sa;
+
+	if (initialized)
+		return;
+
+	/* catch SIGINT */
+	sa.sa_handler = sigint_signal_handler;
+	sigemptyset (&sa.sa_mask);
+	sa.sa_flags = 0;
+	g_assert (sigaction (SIGINT, &sa, NULL) != -1);
+
+	initialized = TRUE;
+}
+
+static int
+server_ptrace_get_pending_sigint (void)
+{
+	if (pending_sigint > 0)
+		return pending_sigint--;
+
+	return 0;
+}
+
 extern void GC_start_blocking (void);
 extern void GC_end_blocking (void);
 
@@ -637,6 +624,10 @@ extern void GC_end_blocking (void);
 #include "x86-freebsd-ptrace.c"
 #endif
 
+#ifdef __MACH__
+#include "darwin-ptrace.c"
+#endif
+
 #if defined(__i386__)
 #include "i386-arch.c"
 #elif defined(__x86_64__)
@@ -646,7 +637,10 @@ extern void GC_end_blocking (void);
 #endif
 
 InferiorVTable i386_ptrace_inferior = {
+	server_ptrace_static_init,
 	server_ptrace_global_init,
+	server_ptrace_get_server_type,
+	server_ptrace_get_capabilities,
 	server_ptrace_create_inferior,
 	server_ptrace_initialize_process,
 	server_ptrace_initialize_thread,
@@ -696,5 +690,13 @@ InferiorVTable i386_ptrace_inferior = {
 	server_ptrace_push_registers,
 	server_ptrace_pop_registers,
 	server_ptrace_get_callback_frame,
+	server_ptrace_restart_notification,
 	server_ptrace_get_registers_from_core_file,
+	server_ptrace_get_current_pid,
+	server_ptrace_get_current_thread,
+	server_ptrace_sem_init,
+	server_ptrace_sem_wait,
+	server_ptrace_sem_post,
+	server_ptrace_sem_get_value,
+	server_ptrace_get_pending_sigint,
 };
