@@ -32,6 +32,7 @@ struct ArchInfo
 	CodeBufferData *code_buffer;
 	guint64 dr_control, dr_status;
 	guint64 pushed_regs_rsp;
+	BreakpointManager *hw_bpm;
 	int dr_regs [DR_NADDR];
 };
 
@@ -58,6 +59,7 @@ x86_arch_initialize (void)
 	ArchInfo *arch = g_new0 (ArchInfo, 1);
 
 	arch->callback_stack = g_ptr_array_new ();
+	arch->hw_bpm = mono_debugger_breakpoint_manager_new ();
 
 	return arch;
 }
@@ -66,6 +68,7 @@ void
 x86_arch_finalize (ArchInfo *arch)
 {
 	g_ptr_array_free (arch->callback_stack, TRUE);
+	mono_debugger_breakpoint_manager_free (arch->hw_bpm);
 	g_free (arch);
 }
 
@@ -73,7 +76,8 @@ static ServerCommandError
 server_ptrace_current_insn_is_bpt (ServerHandle *handle, guint32 *is_breakpoint)
 {
 	mono_debugger_breakpoint_manager_lock ();
-	if (mono_debugger_breakpoint_manager_lookup (handle->bpm, INFERIOR_REG_RIP (handle->arch->current_regs)))
+	if (mono_debugger_breakpoint_manager_lookup (handle->arch->hw_bpm, INFERIOR_REG_RIP (handle->arch->current_regs)) ||
+	    mono_debugger_breakpoint_manager_lookup (handle->bpm, INFERIOR_REG_RIP (handle->arch->current_regs)))
 		*is_breakpoint = TRUE;
 	else
 		*is_breakpoint = FALSE;
@@ -141,6 +145,35 @@ check_breakpoint (ServerHandle *handle, guint64 address, guint64 *retval)
 	return TRUE;
 }
 
+static BreakpointInfo *
+lookup_breakpoint (ServerHandle *handle, guint32 idx, BreakpointManager **out_bpm)
+{
+	BreakpointInfo *info;
+
+	mono_debugger_breakpoint_manager_lock ();
+	info = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->arch->hw_bpm, idx);
+	if (info) {
+		if (out_bpm)
+			*out_bpm = handle->arch->hw_bpm;
+		mono_debugger_breakpoint_manager_unlock ();
+		return info;
+	}
+
+	info = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, idx);
+	if (info) {
+		if (out_bpm)
+			*out_bpm = handle->bpm;
+		mono_debugger_breakpoint_manager_unlock ();
+		return info;
+	}
+
+	if (out_bpm)
+		*out_bpm = NULL;
+
+	mono_debugger_breakpoint_manager_unlock ();
+	return info;
+}
+
 static CallbackData *
 get_callback_data (ArchInfo *arch)
 {
@@ -187,8 +220,18 @@ x86_arch_child_stopped (ServerHandle *handle, int stopsig,
 	if (stopsig == SIGSTOP)
 		return STOP_ACTION_INTERRUPTED;
 
+	/*
+	 * By default, when the NX-flag is set in the BIOS, we stop at the `cdata->call_address'
+	 * (which contains an `int 3' instruction) with a SIGSEGV.
+	 *
+	 * When the NX-flag is turned off, that `int 3' instruction is actually executed and we
+	 * stop normally.
+	 */
+
 	cdata = get_callback_data (arch);
-	if (cdata && (cdata->call_address == INFERIOR_REG_RIP (arch->current_regs))) {
+	if (cdata &&
+	    (((stopsig == SIGSEGV) && (cdata->call_address == INFERIOR_REG_RIP (arch->current_regs))) ||
+	     (cdata->call_address == INFERIOR_REG_RIP (arch->current_regs) - 1))) {
 		if (cdata->pushed_registers) {
 			guint64 pushed_regs [13];
 
@@ -679,15 +722,6 @@ server_ptrace_insert_breakpoint (ServerHandle *handle, guint64 address, guint32
 	mono_debugger_breakpoint_manager_lock ();
 	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup (handle->bpm, address);
 	if (breakpoint) {
-		/*
-		 * You cannot have a hardware breakpoint and a normal breakpoint on the same
-		 * instruction.
-		 */
-		if (breakpoint->is_hardware_bpt) {
-			mono_debugger_breakpoint_manager_unlock ();
-			return COMMAND_ERROR_DR_OCCUPIED;
-		}
-
 		breakpoint->refcount++;
 		goto done;
 	}
@@ -717,13 +751,14 @@ server_ptrace_insert_breakpoint (ServerHandle *handle, guint64 address, guint32
 }
 
 static ServerCommandError
-server_ptrace_remove_breakpoint (ServerHandle *handle, guint32 bhandle)
+server_ptrace_remove_breakpoint (ServerHandle *handle, guint32 idx)
 {
+	BreakpointManager *bpm;
 	BreakpointInfo *breakpoint;
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, bhandle);
+	breakpoint = lookup_breakpoint (handle, idx, &bpm);
 	if (!breakpoint) {
 		result = COMMAND_ERROR_NO_SUCH_BREAKPOINT;
 		goto out;
@@ -739,7 +774,7 @@ server_ptrace_remove_breakpoint (ServerHandle *handle, guint32 bhandle)
 		goto out;
 
 	breakpoint->enabled = FALSE;
-	mono_debugger_breakpoint_manager_remove (handle->bpm, (BreakpointInfo *) breakpoint);
+	mono_debugger_breakpoint_manager_remove (bpm, breakpoint);
 
  out:
 	mono_debugger_breakpoint_manager_unlock ();
@@ -769,11 +804,6 @@ server_ptrace_insert_hw_breakpoint (ServerHandle *handle, guint32 type, guint32
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup (handle->bpm, address);
-	if (breakpoint) {
-		breakpoint->refcount++;
-		goto done;
-	}
 
 	result = find_free_hw_register (handle, idx);
 	if (result != COMMAND_ERROR_NONE) {
@@ -797,7 +827,8 @@ server_ptrace_insert_hw_breakpoint (ServerHandle *handle, guint32 type, guint32
 	}
 
 	breakpoint->enabled = TRUE;
-	mono_debugger_breakpoint_manager_insert (handle->bpm, (BreakpointInfo *) breakpoint);
+	mono_debugger_breakpoint_manager_insert (handle->arch->hw_bpm, (BreakpointInfo *) breakpoint);
+
  done:
 	*bhandle = breakpoint->id;
 	mono_debugger_breakpoint_manager_unlock ();
@@ -806,13 +837,13 @@ server_ptrace_insert_hw_breakpoint (ServerHandle *handle, guint32 type, guint32
 }
 
 static ServerCommandError
-server_ptrace_enable_breakpoint (ServerHandle *handle, guint32 bhandle)
+server_ptrace_enable_breakpoint (ServerHandle *handle, guint32 idx)
 {
 	BreakpointInfo *breakpoint;
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, bhandle);
+	breakpoint = lookup_breakpoint (handle, idx, NULL);
 	if (!breakpoint) {
 		mono_debugger_breakpoint_manager_unlock ();
 		return COMMAND_ERROR_NO_SUCH_BREAKPOINT;
@@ -825,13 +856,13 @@ server_ptrace_enable_breakpoint (ServerHandle *handle, guint32 bhandle)
 }
 
 static ServerCommandError
-server_ptrace_disable_breakpoint (ServerHandle *handle, guint32 bhandle)
+server_ptrace_disable_breakpoint (ServerHandle *handle, guint32 idx)
 {
 	BreakpointInfo *breakpoint;
 	ServerCommandError result;
 
 	mono_debugger_breakpoint_manager_lock ();
-	breakpoint = (BreakpointInfo *) mono_debugger_breakpoint_manager_lookup_by_id (handle->bpm, bhandle);
+	breakpoint = lookup_breakpoint (handle, idx, NULL);
 	if (!breakpoint) {
 		mono_debugger_breakpoint_manager_unlock ();
 		return COMMAND_ERROR_NO_SUCH_BREAKPOINT;
@@ -1160,13 +1191,13 @@ server_ptrace_call_method_invoke (ServerHandle *handle, guint64 invoke_method,
 			ptr [i] = param_data [i];
 	}
 
-	*((guint64 *) code) = new_rsp + static_size - 1;
+	*((guint64 *) code) = new_rsp + 24;
 	*((guint64 *) (code+8)) = callback_argument;
 
 	cdata = g_new0 (CallbackData, 1);
 	memcpy (&cdata->saved_regs, &arch->current_regs, sizeof (arch->current_regs));
 	memcpy (&cdata->saved_fpregs, &arch->current_fpregs, sizeof (arch->current_fpregs));
-	cdata->call_address = new_rsp + static_size - 1;
+	cdata->call_address = new_rsp + 24;
 	cdata->stack_pointer = new_rsp + 8;
 	cdata->exc_address = new_rsp + 16;
 	cdata->callback_argument = callback_argument;
@@ -1380,3 +1411,16 @@ server_ptrace_get_callback_frame (ServerHandle *handle, guint64 stack_pointer,
 
 	return COMMAND_ERROR_NO_CALLBACK_FRAME;
 }
+
+static ServerCommandError
+server_ptrace_restart_notification (ServerHandle *handle)
+{
+	ServerCommandError result;
+
+	if (!handle->mono_runtime ||
+	    (INFERIOR_REG_RIP (handle->arch->current_regs) - 1 != handle->mono_runtime->notification_address))
+		return COMMAND_ERROR_INTERNAL_ERROR;
+
+	INFERIOR_REG_RIP (handle->arch->current_regs)--;
+	return _server_ptrace_set_registers (handle->inferior, &handle->arch->current_regs);
+}
