@@ -32,6 +32,8 @@ namespace Mono.Debugger
 
 	public interface IExpressionParser
 	{
+		IExpressionParser Clone (DebuggerSession session);
+
 		SourceLocation ParseLocation (Thread target, StackFrame frame,
 					      LocationType type, string name);
 	}
@@ -61,12 +63,10 @@ namespace Mono.Debugger
 		IExpressionParser parser;
 		XmlDocument saved_session;
 
-		private DebuggerSession (DebuggerConfiguration config, string name,
-					 IExpressionParser parser)
+		private DebuggerSession (DebuggerConfiguration config, string name)
 		{
 			this.Config = config;
 			this.Name = name;
-			this.parser = parser;
 
 			modules = Hashtable.Synchronized (new Hashtable ());
 			displays = Hashtable.Synchronized (new Hashtable ());
@@ -82,16 +82,41 @@ namespace Mono.Debugger
 
 		public DebuggerSession (DebuggerConfiguration config, DebuggerOptions options,
 					string name, IExpressionParser parser)
-			: this (config, name, parser)
+			: this (config, name)
 		{
 			this.Options = options;
+			this.parser = parser;
 
-			AddEvent (new MainMethodBreakpoint (this));
+			if (config.IsCLI)
+				AddEvent (new MainMethodBreakpoint (this));
 		}
 
-		internal DebuggerSession Clone (DebuggerOptions new_options, string new_name)
+		protected DebuggerSession (DebuggerConfiguration config, Process process, DebuggerOptions options,
+					   string name, IExpressionParser parser, XPathNavigator nav)
+			: this (config, name)
 		{
-			return new DebuggerSession (Config, new_options, new_name, parser);
+			this.main_process = process;
+			this.Options = options;
+			this.parser = parser.Clone (this);
+
+			XPathNodeIterator event_iter = nav.Select ("Events/*");
+			LoadEvents (event_iter);
+
+			XPathNodeIterator display_iter = nav.Select ("Displays/*");
+			LoadDisplays (display_iter);
+		}
+
+		internal DebuggerSession Clone (Process new_process, DebuggerOptions new_options, string new_name)
+		{
+			XmlDocument document = SaveSession ();
+
+			XPathNavigator nav = document.CreateNavigator ();
+			XPathNodeIterator session_iter = nav.Select (
+				"/DebuggerConfiguration/DebuggerSession[@name='" + Name + "']");
+			if (!session_iter.MoveNext ())
+				throw new InternalError ();
+
+			return new DebuggerSession (Config, new_process, new_options, new_name, parser, session_iter.Current);
 		}
 
 		public void SaveSession (Stream stream)
@@ -147,8 +172,10 @@ namespace Mono.Debugger
 
 		public DebuggerSession (DebuggerConfiguration config, Stream stream,
 					IExpressionParser parser)
-			: this (config, "main", parser)
+			: this (config, "main")
 		{
+			this.parser = parser;
+
 			XmlReaderSettings settings = new XmlReaderSettings ();
 			Assembly ass = Assembly.GetExecutingAssembly ();
 			using (Stream schema = ass.GetManifestResourceStream ("DebuggerConfiguration"))
@@ -262,9 +289,7 @@ namespace Mono.Debugger
 			if (main_process == null)
 				return null;
 
-			Module[] modules = main_process.Modules;
-
-			foreach (Module module in modules) {
+			foreach (Module module in modules.Values) {
 				SourceFile file = module.FindFile (filename);
 				if (file != null)
 					return file;
@@ -276,7 +301,7 @@ namespace Mono.Debugger
 			filename = Path.GetFullPath (Path.Combine (
 				Options.WorkingDirectory, filename));
 
-			foreach (Module module in modules) {
+			foreach (Module module in modules.Values) {
 				SourceFile file = module.FindFile (filename);
 				if (file != null)
 					return file;
@@ -307,11 +332,15 @@ namespace Mono.Debugger
 			if (process != main_process)
 				return;
 
-			pending_bpts.Clear ();
-			reached_main = false;
-			main_process = null;
-
 			lock (this) {
+				pending_bpts.Clear ();
+				reached_main = false;
+				main_process = null;
+
+				foreach (Module module in modules.Values) {
+					module.UnLoadModule ();
+				}
+
 				foreach (Event e in events.Values.ToArray ()) {
 					e.OnTargetExited ();
 					if (!e.IsPersistent)
@@ -320,6 +349,15 @@ namespace Mono.Debugger
 			}
 		}
 
+		internal void OnProcessExecd (Process process)
+		{
+			if (process != main_process)
+				return;
+
+			OnProcessExited (process);
+			main_process = process;
+		}
+
 		protected void LoadSession (XPathNavigator nav)
 		{
 			XPathNodeIterator session_iter = nav.Select (
@@ -352,6 +390,14 @@ namespace Mono.Debugger
 			}
 
 			XPathNodeIterator event_iter = session_iter.Current.Select ("Events/*");
+			LoadEvents (event_iter);
+
+			XPathNodeIterator display_iter = session_iter.Current.Select ("Displays/*");
+			LoadDisplays (display_iter);
+		}
+
+		protected void LoadEvents (XPathNodeIterator event_iter)
+		{
 			while (event_iter.MoveNext ()) {
 				if (event_iter.Current.Name != "Breakpoint")
 					throw new InternalError ();
@@ -382,8 +428,10 @@ namespace Mono.Debugger
 				e.IsEnabled = enabled;
 				AddEvent (e);
 			}
+		}
 
-			XPathNodeIterator display_iter = session_iter.Current.Select ("Displays/*");
+		protected void LoadDisplays (XPathNodeIterator display_iter)
+		{
 			while (display_iter.MoveNext ()) {
 				if (display_iter.Current.Name != "Display")
 					throw new InternalError ();
@@ -397,6 +445,7 @@ namespace Mono.Debugger
 			}
 		}
 
+
 		protected Event ParseEvent (XPathNavigator navigator, int index, ThreadGroup group)
 		{
 			if (navigator.Name == "Location") {
@@ -417,6 +466,7 @@ namespace Mono.Debugger
 				throw new InternalError ();
 		}
 
+
 		//
 		// Modules.
 		//
@@ -458,20 +508,22 @@ namespace Mono.Debugger
 			if (symfile == null)
 				throw new NullReferenceException ();
 
-			Module module = (Module) modules [name];
-			if (module != null)
+			lock (this) {
+				Module module = (Module) modules [name];
+				if (module != null)
+					return module;
+
+				ModuleGroup group;
+				if (is_user_module (symfile.CodeBase))
+					group = Config.GetModuleGroup ("user");
+				else
+					group = Config.GetModuleGroup (symfile);
+
+				module = new Module (group, name, symfile);
+				modules.Add (name, module);
+
 				return module;
-
-			ModuleGroup group;
-			if (is_user_module (symfile.CodeBase))
-				group = Config.GetModuleGroup ("user");
-			else
-				group = Config.GetModuleGroup (symfile);
-
-			module = new Module (group, name, symfile);
-			modules.Add (name, module);
-
-			return module;
+			}
 		}
 
 		public Module[] Modules {
@@ -543,7 +595,10 @@ namespace Mono.Debugger
 		public Event GetEvent (int index)
 		{
 			lock (this) {
-				return (Event) events [index];
+				if (!events.ContainsKey (index))
+					return null;
+
+				return events [index];
 			}
 		}
 
