@@ -157,7 +157,7 @@ namespace Mono.Debugger
 				return EvaluationResult.Ok;
 			}
 
-			obj = obj.GetParentObject (thread) as TargetStructObject;
+			obj = obj.GetParentObject (thread) as TargetClassObject;
 			if (obj != null)
 				goto again;
 
