@@ -105,7 +105,7 @@ namespace Mono.Debugger.Frontend
 			if (obj == null)
 				return null;
 
-			TargetStructObject cobj = obj as TargetStructObject;
+			TargetClassObject cobj = obj as TargetClassObject;
 			if (cobj != null) {
 				TargetObject current;
 				try {
@@ -763,14 +763,14 @@ namespace Mono.Debugger.Frontend
 			if (method == null)
 				return null;
 
-			TargetStructType decl_type = method.GetDeclaringType (context.CurrentThread);
+			TargetClassType decl_type = method.GetDeclaringType (context.CurrentThread);
 			if (decl_type == null)
 				return null;
 
-			TargetStructObject instance = null;
+			TargetClassObject instance = null;
 			if (method.HasThis) {
 				TargetVariable this_var = method.GetThis (context.CurrentThread);
-				instance = (TargetStructObject) this_var.GetObject (frame);
+				instance = (TargetClassObject) this_var.GetObject (frame);
 			}
 
 			member = StructAccessExpression.FindMember (
@@ -1003,7 +1003,7 @@ namespace Mono.Debugger.Frontend
 			}
 
 			if (lexpr != null) {
-				TargetStructObject sobj = Convert.ToStructObject (
+				TargetClassObject sobj = Convert.ToStructObject (
 					target, lexpr.EvaluateObject (context));
 				if (sobj == null)
 					throw new ScriptingException (
@@ -2323,10 +2323,10 @@ namespace Mono.Debugger.Frontend
 			return this;
 		}
 
-		static TargetStructObject TryParentCast (ScriptingContext context,
-							 TargetStructObject source,
-							 TargetStructType source_type,
-							 TargetStructType target_type)
+		static TargetClassObject TryParentCast (ScriptingContext context,
+							 TargetClassObject source,
+							 TargetClassType source_type,
+							 TargetClassType target_type)
 		{
 			if (source_type == target_type)
 				return source;
@@ -2334,7 +2334,7 @@ namespace Mono.Debugger.Frontend
 			if (!source_type.HasParent)
 				return null;
 
-			TargetStructType parent_type = source_type.GetParentType (context.CurrentThread);
+			TargetClassType parent_type = source_type.GetParentType (context.CurrentThread);
 			source = TryParentCast (context, source, parent_type, target_type);
 			if (source == null)
 				return null;
@@ -2342,11 +2342,11 @@ namespace Mono.Debugger.Frontend
 			return source.GetParentObject (context.CurrentThread) as TargetClassObject;
 		}
 
-		static TargetStructObject TryCurrentCast (ScriptingContext context,
+		static TargetClassObject TryCurrentCast (ScriptingContext context,
 							  TargetClassObject source,
 							  TargetClassType target_type)
 		{
-			TargetStructObject current = source.GetCurrentObject (context.CurrentThread);
+			TargetClassObject current = source.GetCurrentObject (context.CurrentThread);
 			if (current == null)
 				return null;
 
@@ -2363,7 +2363,7 @@ namespace Mono.Debugger.Frontend
 			if (sobj == null)
 				return null;
 
-			TargetStructObject result = TryParentCast (context, sobj, sobj.Type, target_type);
+			TargetClassObject result = TryParentCast (context, sobj, sobj.Type, target_type);
 			if (result != null)
 				return result;
 
@@ -2371,8 +2371,8 @@ namespace Mono.Debugger.Frontend
 		}
 
 		static bool TryParentCast (ScriptingContext context,
-					   TargetStructType source_type,
-					   TargetStructType target_type)
+					   TargetClassType source_type,
+					   TargetClassType target_type)
 		{
 			if (source_type == target_type)
 				return true;
@@ -2380,7 +2380,7 @@ namespace Mono.Debugger.Frontend
 			if (!source_type.HasParent)
 				return false;
 
-			TargetStructType parent_type = source_type.GetParentType (context.CurrentThread);
+			TargetClassType parent_type = source_type.GetParentType (context.CurrentThread);
 			return TryParentCast (context, parent_type, target_type);
 		}
 
@@ -2421,6 +2421,9 @@ namespace Mono.Debugger.Frontend
 				    (source is TargetFundamentalObject))
 					return target_type.Language.CreateBoxedObject (context.CurrentThread, source);
 
+				if (source is TargetObjectObject)
+					return source;
+
 				throw new ScriptingException (
 					"Cannot box object `{0}': not a value-type", expr.Name);
 			}
@@ -2657,8 +2660,8 @@ namespace Mono.Debugger.Frontend
 		}
 
 		static bool ImplicitReferenceConversionExists (ScriptingContext context,
-							       TargetStructType source,
-							       TargetStructType target)
+							       TargetClassType source,
+							       TargetClassType target)
 		{
 			if (source == target)
 				return true;
@@ -2666,7 +2669,7 @@ namespace Mono.Debugger.Frontend
 			if (!source.HasParent)
 				return false;
 
-			TargetStructType parent_type = source.GetParentType (context.CurrentThread);
+			TargetClassType parent_type = source.GetParentType (context.CurrentThread);
 			return ImplicitReferenceConversionExists (context, parent_type, target);
 		}
 
@@ -2694,6 +2697,10 @@ namespace Mono.Debugger.Frontend
 					context, (TargetFundamentalType) source,
 					(TargetFundamentalType) target);
 
+			if ((source is TargetFundamentalType) && (target is TargetObjectType) &&
+			    (((TargetFundamentalType) source).FundamentalKind == FundamentalKind.String))
+				return true;
+
 			if ((source is TargetClassType) && (target is TargetClassType))
 				return ImplicitReferenceConversionExists (
 					context, (TargetClassType) source,
@@ -2713,6 +2720,10 @@ namespace Mono.Debugger.Frontend
 					context, (TargetFundamentalObject) obj,
 					(TargetFundamentalType) type);
 
+			if ((obj is TargetFundamentalObject) && (type is TargetObjectType) &&
+			    (((TargetFundamentalType) obj.Type).FundamentalKind == FundamentalKind.String))
+				return obj.Type.Language.CreateBoxedObject (context.CurrentThread, obj);
+
 			if ((obj is TargetClassObject) && (type is TargetClassType))
 				return ImplicitReferenceConversion (
 					context, (TargetClassObject) obj,
@@ -2784,9 +2795,9 @@ namespace Mono.Debugger.Frontend
 			return null;
 		}
 
-		public static TargetStructObject ToStructObject (Thread target, TargetObject obj)
+		public static TargetClassObject ToStructObject (Thread target, TargetObject obj)
 		{
-			TargetStructObject sobj = obj as TargetStructObject;
+			TargetClassObject sobj = obj as TargetClassObject;
 			if (sobj != null)
 				return sobj;
 
@@ -3059,6 +3070,8 @@ namespace Mono.Debugger.Frontend
 			} catch (TargetException ex) {
 				throw new ScriptingException (
 					"Invocation of `{0}' raised an exception: {1}", Name, ex.Message);
+			} catch (EvaluationTimeoutException ex) {
+				throw new ScriptingException ("Invocation of `{0}' timed out.", Name);
 			}
 		}
 
@@ -3219,7 +3232,7 @@ namespace Mono.Debugger.Frontend
 			if (obj == null)
 				return null;
 
-			TargetStructObject sobj = (TargetStructObject) obj;
+			TargetClassObject sobj = (TargetClassObject) obj;
 			for (int i = 0; i <= level; i++) {
 				sobj = sobj.GetParentObject (context.CurrentThread);
 				if (sobj == null)
