@@ -21,6 +21,10 @@ namespace Mono.Debugger.Frontend
 			}
 		}
 
+		public DebuggerConfiguration Configuration {
+			get; private set;
+		}
+
 		public Interpreter Interpreter {
 			get { return interpreter; }
 		}
@@ -29,8 +33,8 @@ namespace Mono.Debugger.Frontend
 			get { return interrupt_event; }
 		}
 
-		public ST.AutoResetEvent EnterNestedBreakStateEvent {
-			get { return enter_nested_break_state_event; }
+		public ST.AutoResetEvent NestedBreakStateEvent {
+			get { return nested_break_state_event; }
 		}
 
 		Interpreter interpreter;
@@ -43,7 +47,7 @@ namespace Mono.Debugger.Frontend
 		ST.Thread main_thread;
 
 		ST.AutoResetEvent interrupt_event;
-		ST.AutoResetEvent enter_nested_break_state_event;
+		ST.AutoResetEvent nested_break_state_event;
 
 		Stack<CommandLineInterpreter.MainLoop> main_loop_stack;
 
@@ -58,22 +62,33 @@ namespace Mono.Debugger.Frontend
 			mono_debugger_server_static_init ();
 		}
 
-		internal CommandLineInterpreter (bool is_interactive, DebuggerConfiguration config,
-						 DebuggerOptions options)
+		internal CommandLineInterpreter (DebuggerOptions options, bool is_interactive)
 		{
 			if (options.HasDebugFlags)
 				Report.Initialize (options.DebugOutput, options.DebugFlags);
 			else
 				Report.Initialize ();
 
-			interpreter = new Interpreter (is_interactive, config, options);
+			Configuration = new DebuggerConfiguration ();
+#if HAVE_XSP
+			if (options.StartXSP)
+				Configuration.SetupXSP ();
+			else
+				Configuration.LoadConfiguration ();
+#else
+			Configuration.LoadConfiguration ();
+#endif
+
+			Configuration.SetupCLI ();
+
+			interpreter = new Interpreter (is_interactive, Configuration, options);
 			interpreter.CLI = this;
 
 			engine = interpreter.DebuggerEngine;
 			parser = new LineParser (engine);
 
 			interrupt_event = new ST.AutoResetEvent (false);
-			enter_nested_break_state_event = new ST.AutoResetEvent (false);
+			nested_break_state_event = new ST.AutoResetEvent (false);
 
 			main_loop_stack = new Stack<MainLoop> ();
 			main_loop_stack.Push (new MainLoop (interpreter));
@@ -94,7 +109,7 @@ namespace Mono.Debugger.Frontend
 			parser = new LineParser (engine);
 
 			interrupt_event = new ST.AutoResetEvent (false);
-			enter_nested_break_state_event = new ST.AutoResetEvent (false);
+			nested_break_state_event = new ST.AutoResetEvent (false);
 
 			main_loop_stack = new Stack<MainLoop> ();
 			main_loop_stack.Push (new MainLoop (interpreter));
@@ -163,7 +178,7 @@ namespace Mono.Debugger.Frontend
 			Report.Debug (DebugFlags.CLI, "{0} waiting for completion", loop);
 
 			ST.WaitHandle[] wait = new ST.WaitHandle[] {
-				loop.CompletedEvent, interrupt_event, enter_nested_break_state_event
+				loop.CompletedEvent, interrupt_event, nested_break_state_event
 			};
 			int ret = ST.WaitHandle.WaitAny (wait);
 
@@ -182,7 +197,7 @@ namespace Mono.Debugger.Frontend
 
 			Report.Debug (DebugFlags.CLI, "{0} enter nested break state", loop);
 
-			enter_nested_break_state_event.Set ();
+			nested_break_state_event.Set ();
 		}
 
 		public void LeaveNestedBreakState ()
@@ -191,6 +206,8 @@ namespace Mono.Debugger.Frontend
 			nested.Completed = true;
 
 			Report.Debug (DebugFlags.CLI, "{0} leave nested break state", nested);
+
+			nested_break_state_event.Set ();
 		}
 
 		void main_thread_main ()
@@ -293,11 +310,11 @@ namespace Mono.Debugger.Frontend
 				Console.Write (the_prompt);
 				return Console.ReadLine ();
 			} else {
-				string result = GnuReadLine.ReadLine (the_prompt);
+				string result = LineReader.ReadLine (the_prompt);
 				if (result == null)
 					return null;
 				if (result != "")
-					GnuReadLine.AddHistory (result);
+					LineReader.AddHistory (result);
 				return result;
 			}
 		}
@@ -332,28 +349,17 @@ namespace Mono.Debugger.Frontend
 
 		public static void Main (string[] args)
 		{
-			bool is_terminal = GnuReadLine.IsTerminal (0);
+			bool is_terminal = LineReader.IsTerminal (0);
 
 			DebuggerOptions options = DebuggerOptions.ParseCommandLine (args);
 
-			DebuggerConfiguration config = new DebuggerConfiguration ();
-#if HAVE_XSP
-			if (options.StartXSP)
-				config.SetupXSP ();
-			else
-				config.LoadConfiguration ();
-#else
-			config.LoadConfiguration ();
-#endif
-
 			Console.WriteLine ("Mono Debugger");
 
-			CommandLineInterpreter interpreter = new CommandLineInterpreter (
-				is_terminal, config, options);
+			CommandLineInterpreter interpreter = new CommandLineInterpreter (options, is_terminal);
 
 			interpreter.RunMainLoop ();
 
-			config.SaveConfiguration ();
+			interpreter.Configuration.SaveConfiguration ();
 		}
 
 		public class MainLoop
