@@ -286,7 +286,6 @@ namespace Mono.Debugger.Frontend
 				throw new EvaluationTimeoutException ();
 			}
 
-			Interpreter.CheckLastEvent (thread);
 			return result;
 		}
 
@@ -417,9 +416,9 @@ namespace Mono.Debugger.Frontend
 				thread, instance, attr.Type, timeout, out type);
 		}
 
-		string MonoObjectToString (TargetStructObject obj)
+		string MonoObjectToString (TargetClassObject obj)
 		{
-			TargetStructType ctype = obj.Type;
+			TargetClassType ctype = obj.Type;
 			if ((ctype.Name == "System.Object") || (ctype.Name == "System.ValueType"))
 				return null;
 
@@ -444,7 +443,7 @@ namespace Mono.Debugger.Frontend
 			return null;
 		}
 
-		TargetStructObject CheckTypeProxy (TargetStructObject obj)
+		TargetClassObject CheckTypeProxy (TargetStructObject obj)
 		{
 			if (obj.Type.DebuggerTypeProxyAttribute == null)
 				return null;
@@ -461,14 +460,14 @@ namespace Mono.Debugger.Frontend
 				if (expression == null)
 					return null;
 
-				return (TargetStructObject) expression.EvaluateObject (this);
+				return (TargetClassObject) expression.EvaluateObject (this);
 			} catch {
 				return null;
 			}
 		}
 
-		public static TargetStructObject CheckTypeProxy (Interpreter interpreter, Thread thread,
-								 TargetStructObject obj)
+		public static TargetClassObject CheckTypeProxy (Interpreter interpreter, Thread thread,
+								TargetStructObject obj)
 		{
 			if (obj.Type.DebuggerTypeProxyAttribute == null)
 				return null;
@@ -484,7 +483,7 @@ namespace Mono.Debugger.Frontend
 		string DoFormatObject (TargetObject obj, DisplayFormat format)
 		{
 			if (format == DisplayFormat.Object) {
-				TargetStructObject cobj = obj as TargetStructObject;
+				TargetClassObject cobj = obj as TargetClassObject;
 				if (cobj != null) {
 					string formatted = MonoObjectToString (cobj);
 					if (formatted != null)
@@ -677,5 +676,14 @@ namespace Mono.Debugger.Frontend
 				       ex);
 			}
 		}
+
+		internal void ActivatePendingBreakpoints ()
+		{
+			CommandResult result = Interpreter.CurrentProcess.ActivatePendingBreakpoints ();
+			if (result == null)
+				return;
+
+			Interpreter.Wait (result);
+		}
 	}
 }
