@@ -2,7 +2,7 @@ namespace Mono.Debugger.Languages
 {
 	public abstract class TargetClassObject : TargetStructObject
 	{
-		public readonly new TargetClassType Type;
+		new public TargetClassType Type;
 
 		internal TargetClassObject (TargetClassType type, TargetLocation location)
 			: base (type, location)
