@@ -3,6 +3,7 @@ namespace Mono.Debugger.Languages
 	public enum TargetObjectKind
 	{
 		Unknown,
+		Null,
 		Fundamental,
 		Enum,
 		Array,
@@ -12,6 +13,7 @@ namespace Mono.Debugger.Languages
 		Object,
 		Function,
 		Alias,
+		Nullable,
 		GenericParameter,
 		GenericInstance
 	}
