@@ -2,7 +2,7 @@ namespace Mono.Debugger.Languages
 {
 	public abstract class TargetStructObject : TargetObject
 	{
-		public readonly new TargetStructType Type;
+		new public TargetStructType Type;
 
 		internal TargetStructObject (TargetStructType type, TargetLocation location)
 			: base (type, location)
@@ -10,27 +10,27 @@ namespace Mono.Debugger.Languages
 			this.Type = type;
 		}
 
-		public TargetStructObject GetParentObject (Thread thread)
+		public TargetClassObject GetParentObject (Thread thread)
 		{
-			return (TargetStructObject) thread.ThreadServant.DoTargetAccess (
+			return (TargetClassObject) thread.ThreadServant.DoTargetAccess (
 				delegate (TargetMemoryAccess target) {
 					return GetParentObject (target);
 			});
 		}
 
-		internal abstract TargetStructObject GetParentObject (TargetMemoryAccess target);
+		internal abstract TargetClassObject GetParentObject (TargetMemoryAccess target);
 
-		public TargetStructObject GetCurrentObject (Thread thread)
+		public TargetClassObject GetCurrentObject (Thread thread)
 		{
 			if (!type.IsByRef)
 				return null;
 
-			return (TargetStructObject) thread.ThreadServant.DoTargetAccess (
+			return (TargetClassObject) thread.ThreadServant.DoTargetAccess (
 				delegate (TargetMemoryAccess target) {
 					return GetCurrentObject (target);
 			});
 		}
 
-		internal abstract TargetStructObject GetCurrentObject (TargetMemoryAccess target);
+		internal abstract TargetClassObject GetCurrentObject (TargetMemoryAccess target);
 	}
 }
