@@ -5,7 +5,7 @@ namespace Mono.Debugger.Languages
 	// <summary>
 	//   This interface provides information about a variable in the target application.
 	// </summary>
-	public abstract class TargetVariable : MarshalByRefObject
+	public abstract class TargetVariable : DebuggerMarshalByRefObject
 	{
 		public abstract string Name {
 			get;
@@ -15,6 +15,12 @@ namespace Mono.Debugger.Languages
 			get;
 		}
 
+		// <summary>
+		//   Checks whether the variable is accessible in the lexical scope around
+		//   address @address, but without actually trying to access the variable.
+		// </summary>
+		public abstract bool IsInScope (TargetAddress address);
+
 		// <summary>
 		//   Checks whether the variable is alive at @address, but without actually
 		//   trying to access the variable.  The implementation just checks the data
@@ -33,7 +39,16 @@ namespace Mono.Debugger.Languages
 		//   bound to any particular target location.  This also means that it won't
 		//   get invalid after the target exited.
 		// </remarks>
-		public abstract TargetObject GetObject (StackFrame frame);
+		public TargetObject GetObject (StackFrame frame)
+		{
+			return (TargetObject) frame.Thread.ThreadServant.DoTargetAccess (
+				delegate (TargetMemoryAccess target)  {
+					return GetObject (frame, target);
+			});
+		}
+
+		internal abstract TargetObject GetObject (StackFrame frame,
+							  TargetMemoryAccess target);
 
 		public abstract string PrintLocation (StackFrame frame);
 
