@@ -1,6 +1,9 @@
 using System;
 using Cecil = Mono.Cecil;
 
+using Mono.Debugger.Backend;
+using Mono.Debugger.Backend.Mono;
+
 namespace Mono.Debugger.Languages.Mono
 {
 	internal class MonoGenericParameterType : TargetGenericParameterType
