@@ -1,4 +1,6 @@
 using System;
+using System.IO;
+using System.Collections;
 using System.Runtime.InteropServices;
 
 using Mono.Debugger.Backend;
@@ -7,7 +9,8 @@ namespace Mono.Debugger.Languages.Native
 {
 	internal class NativeLanguage : Language
 	{
-		BfdContainer bfd_container;
+		ProcessServant process;
+		OperatingSystemBackend os;
 		TargetFundamentalType unsigned_type;
 		TargetFundamentalType integer_type;
 		TargetFundamentalType long_type;
@@ -17,11 +20,16 @@ namespace Mono.Debugger.Languages.Native
 		NativeStringType string_type;
 		TargetInfo info;
 
-		public NativeLanguage (BfdContainer bfd_container, TargetInfo info)
+		Hashtable type_hash;
+
+		public NativeLanguage (ProcessServant process, OperatingSystemBackend os, TargetInfo info)
 		{
-			this.bfd_container = bfd_container;
+			this.process = process;
+			this.os = os;
 			this.info = info;
 
+			this.type_hash = Hashtable.Synchronized (new Hashtable ());
+
 			integer_type = new NativeFundamentalType (this, "int", FundamentalKind.Int32, 4);
 			unsigned_type = new NativeFundamentalType (this, "unsigned int", FundamentalKind.UInt32, 4);
 			long_type = new NativeFundamentalType (this, "long", FundamentalKind.Int64, 8);
@@ -31,6 +39,10 @@ namespace Mono.Debugger.Languages.Native
 			string_type = new NativeStringType (this, info.TargetAddressSize);
 		}
 
+		internal OperatingSystemBackend OperatingSystem {
+			get { return os; }
+		}
+
 		public override string Name {
 			get { return "native"; }
 		}
@@ -40,7 +52,7 @@ namespace Mono.Debugger.Languages.Native
 		}
 
 		internal override ProcessServant Process {
-			get { return bfd_container.Process; }
+			get { return process; }
 		}
 
 		public override TargetFundamentalType IntegerType {
@@ -90,7 +102,22 @@ namespace Mono.Debugger.Languages.Native
 
 		public override TargetType LookupType (string name)
 		{
-			return bfd_container.LookupType (name);
+			os.ReadNativeTypes ();
+
+			ITypeEntry entry = (ITypeEntry) type_hash [name];
+			if (entry == null)
+				return null;
+
+			return entry.ResolveType ();
+		}
+
+		public void AddType (ITypeEntry entry)
+		{
+			if (!type_hash.Contains (entry.Name))
+				type_hash.Add (entry.Name, entry);
+
+			if (entry.IsComplete)
+				type_hash [entry.Name] = entry;
 		}
 
 		TargetFundamentalType GetFundamentalType (Type type)
@@ -155,13 +182,14 @@ namespace Mono.Debugger.Languages.Native
 			return false;
 		}
 
-		internal TargetAddress LookupSymbol (string name)
-		{
-			return bfd_container.LookupSymbol (name);
-		}
-
 		private bool disposed = false;
 
+		private void check_disposed ()
+		{
+			if (disposed)
+				throw new ObjectDisposedException ("NativeLanguage");
+		}
+
 		private void Dispose (bool disposing)
 		{
 			lock (this) {
@@ -172,11 +200,6 @@ namespace Mono.Debugger.Languages.Native
 			}
 
 			if (disposing) {
-				if (bfd_container != null) {
-					bfd_container.Dispose ();
-
-					bfd_container = null;
-				}
 			}
 		}
 
