@@ -8,7 +8,7 @@ using Mono.Debugger.Test.Framework;
 
 namespace Mono.Debugger.Tests
 {
-	[TestFixture]
+	[DebuggerTestFixture]
 	public class TestMultiThread2 : DebuggerTestFixture
 	{
 		public TestMultiThread2 ()
@@ -32,7 +32,7 @@ namespace Mono.Debugger.Tests
 			Thread thread = process.MainThread;
 			AssertStopped (thread, "main", "X.Main()");
 
-			AssertExecute ("next");
+			AssertExecute ("next -bg");
 
 			DebuggerEvent e = AssertEvent ();
 
